#!/usr/bin/env python
# -*- coding: utf-8 -*-
import code
import builtins
from pathlib import Path

from rich.console import Console
from prompt_toolkit import PromptSession
from prompt_toolkit.formatted_text import HTML
from prompt_toolkit.auto_suggest import AutoSuggestFromHistory
from prompt_toolkit.completion import WordCompleter
from prompt_toolkit.lexers import PygmentsLexer
from prompt_toolkit.history import FileHistory
from pygments.lexers.python import PythonLexer

from ..aipy import TaskManager, ConfigManager, CONFIG_DIR
from .. import T, set_lang, __version__

class PythonCompleter(WordCompleter):
    def __init__(self, ai):
        names = ['exit()']
        names += [name for name in dir(builtins)]
        names += [f"ai.{attr}" for attr in dir(ai) if not attr.startswith('_')]
        super().__init__(names, ignore_case=True)
    
def main(args):
    console = Console(record=True)
    console.print(f"[bold cyan]🚀 Python use - AIPython ({__version__}) [[green]https://aipy.app[/green]]")

    conf = ConfigManager(args.config_dir)
    conf.check_config()
    settings = conf.get_config()

    lang = settings.get('lang')
    if lang: set_lang(lang)
    
    settings.gui = False
    settings.debug = args.debug

    try:
        ai = TaskManager(settings, console=console)
    except Exception as e:
        console.print_exception(e)
        return

    update = ai.get_update(True)
    if update and update.get('has_update'):
        console.print(f"[bold red]🔔 号外❗ {T('Update available')}: {update.get('latest_version')}")

    if not ai.client_manager:
        console.print(f"[bold red]{T('No available LLM, please check the configuration file')}")
        return
    
    names = ai.client_manager.names
    console.print(f"{T('Please use ai(task) to enter the task to be processed by AI (enter ai.use(llm) to switch to the following LLM:')}", style="green")
    console.print(f"[cyan]{T('Default')}: [green]{names['default']}，[cyan]{T('Enabled')}: [yellow]{' '.join(names['enabled'])}")

    interp = code.InteractiveConsole({'ai': ai})

    completer = PythonCompleter(ai)
    lexer = PygmentsLexer(PythonLexer)
    auto_suggest = AutoSuggestFromHistory()
    history = FileHistory(str(CONFIG_DIR / '.history.py'))
    session = PromptSession(history=history, completer=completer, lexer=lexer, auto_suggest=auto_suggest)
    while True:
        try:
            user_input = session.prompt(HTML('<ansiblue>>> </ansiblue>'))
            if user_input.strip() in {"exit()", "quit()"}:
                break
            interp.push(user_input)
        except EOFError:
            console.print("[bold yellow]Exiting...")
            break
        except Exception as e:
            console.print(f"[bold red]Error: {e}")



#!/usr/bin/env python
#coding: utf-8

import os
import sys
import time
import json
import queue
import base64
import mimetypes
import traceback
import threading

import wx
import wx.html2
import matplotlib
import matplotlib.pyplot as plt
from loguru import logger
from rich.console import Console
from wx.lib.newevent import NewEvent
from wx.lib.agw.hyperlink import HyperLinkCtrl
from wx import FileDialog, FD_SAVE, FD_OVERWRITE_PROMPT

from .. import __version__, T, set_lang, get_lang, __respath__
from ..aipy.config import ConfigManager, CONFIG_DIR
from ..aipy import TaskManager, event_bus
from . import ConfigDialog, ApiMarketDialog, show_provider_config, AboutDialog, CStatusBar
from ..config import LLMConfig

ChatEvent, EVT_CHAT = NewEvent()
matplotlib.use('Agg')

def image_to_base64(file_path):
    mime_type, _ = mimetypes.guess_type(file_path)
    if mime_type is None:
        return None

    try:
        with open(file_path, "rb") as image_file:
            encoded_string = base64.b64encode(image_file.read()).decode("utf-8")
    except Exception as e:
        return None

    data_url = f"data:{mime_type};base64,{encoded_string}"
    return data_url

class AIPython(threading.Thread):
    def __init__(self, gui):
        super().__init__(daemon=True)
        self.gui = gui
        self.tm = gui.tm
        self._task = None
        self._busy = threading.Event()
        plt.show = self.on_plt_show
        sys.modules["matplotlib.pyplot"] = plt
        self.log = logger.bind(src='aipython')

    def stop_task(self):
        if self._task:
            self._task.stop()
        else:
            self.log.warning("没有正在进行的任务")

    def has_task(self):
        return self._task is not None

    def can_done(self):
        return not self._busy.is_set() and self.has_task()

    def on_plt_show(self, *args, **kwargs):
        filename = f'{time.strftime("%Y%m%d_%H%M%S")}.png'
        plt.savefig(filename)
        user = 'BB-8'
        content = f'![{filename}]({filename})'
        evt = ChatEvent(user=user, msg=content)
        wx.PostEvent(self.gui, evt)

    def on_display(self, image):
        user = T("Turing")
        if image['path']:
            base64_data = image_to_base64(image['path'])
            content = base64_data if base64_data else image['path']
        else:
            content = image['url']

        msg = f'![图片]({content})'
        evt = ChatEvent(user=user, msg=msg)
        wx.PostEvent(self.gui, evt)

    def on_response_complete(self, msg):
        user = T("Turing") #msg['llm']
        #content = f"```markdown\n{msg['content']}\n```"
        evt = ChatEvent(user=user, msg=msg['content'])
        wx.PostEvent(self.gui, evt)

    def on_summary(self, summary):
        user = T("AIPy")
        evt = ChatEvent(user=user, msg=f'{T("End processing instruction")} {summary}')
        wx.PostEvent(self.gui, evt)

    def on_exec(self, block):
        user = 'BB-8'
        content = f"```{block.lang}\n{block.code}\n```"
        evt = ChatEvent(user=user, msg=content)
        wx.PostEvent(self.gui, evt)

    def on_result(self, result):
        user = 'BB-8'
        content = json.dumps(result, indent=4, ensure_ascii=False)
        content = f'{T("Run result")}\n```json\n{content}\n```'
        evt = ChatEvent(user=user, msg=content)
        wx.PostEvent(self.gui, evt)

    def run(self):
        event_bus.register("response_stream", self.on_response_complete)
        event_bus.register("exec", self.on_exec)
        event_bus.register("result", self.on_result)
        event_bus.register("summary", self.on_summary)
        event_bus.register("display", self.on_display)
        while True:
            instruction = self.gui.get_task()
            if instruction in ('/done', 'done'):
                if self._task:
                    self._task.done()
                    self._task = None
                else:
                    self.log.warning("没有正在进行的任务")
                wx.CallAfter(self.gui.on_task_done)
            elif instruction in ('/exit', 'exit'):
                break
            else:
                self._busy.set()
                try:
                    if not self._task:
                        self._task = self.tm.new_task()
                    self._task.run(instruction)
                except Exception as e:
                    self.log.exception('Error running task')
                finally:
                    self._busy.clear()
                wx.CallAfter(self.gui.toggle_input)

class FileDropTarget(wx.FileDropTarget):
    def __init__(self, text_ctrl):
        super().__init__()
        self.text_ctrl = text_ctrl

    def OnDropFiles(self, x, y, filenames):
        s = json.dumps(filenames, ensure_ascii=False)
        self.text_ctrl.AppendText(s)
        return True

class ChatFrame(wx.Frame):
    def __init__(self, tm):
        title = T("🐙 AIPY - Your AI Assistant 🐂 🐎")
        super().__init__(None, title=title, size=(1024, 768))
        
        self.tm = tm
        self.title = title
        self.log = logger.bind(src='gui')
        self.task_queue = queue.Queue()
        self.aipython = AIPython(self)
        self.welcomed = False  # 添加初始化标志
        self.html_file_path = os.path.abspath(__respath__ / f"chatroom_{get_lang()}.html")
        self.avatars = {T("Me"): '🧑', 'BB-8': '🤖', T("Turing"): '🧠', T("AIPy"): '🐙'}

        icon = wx.Icon(str(__respath__ / "aipy.ico"), wx.BITMAP_TYPE_ICO)
        self.SetIcon(icon)

        self.make_menu_bar()
        self.make_panel()
        self.statusbar = CStatusBar(self)
        self.SetStatusBar(self.statusbar)
        self.statusbar.SetStatusText(T("Press Ctrl/Cmd+Enter to send message"), 0)

        self.Bind(EVT_CHAT, self.on_chat)
        self.webview.Bind(wx.html2.EVT_WEBVIEW_TITLE_CHANGED, self.on_webview_title_changed)
        self.aipython.start()
        self.Show()

    def make_input_panel(self, panel):
        self.container = wx.Panel(panel)
 
        self.input = wx.TextCtrl(self.container, style=wx.TE_MULTILINE)
        self.input.SetMinSize((-1, 60))
        self.input.SetWindowStyleFlag(wx.BORDER_SIMPLE)
        self.input.Bind(wx.EVT_KEY_DOWN, self.on_key_down)

        self.done_button = wx.Button(self.container, label=T("End"), size=(50, -1))
        self.done_button.Hide()
        self.done_button.Bind(wx.EVT_BUTTON, self.on_done)
        self.send_button = wx.Button(self.container, label=T("Send"), size=(50, -1))
        self.send_button.Bind(wx.EVT_BUTTON, self.on_send)
        self.container.Bind(wx.EVT_SIZE, self.on_container_resize)
        return self.container

    def make_input_panel2(self, panel):
        container = wx.Panel(panel)
        hbox = wx.BoxSizer(wx.HORIZONTAL)
        self.input = wx.TextCtrl(container, style=wx.TE_MULTILINE)
        self.input.SetMinSize((-1, 80))
        self.input.SetWindowStyleFlag(wx.BORDER_SIMPLE)
        self.input.Bind(wx.EVT_KEY_DOWN, self.on_key_down)
        hbox.Add(self.input, proportion=1, flag=wx.EXPAND | wx.ALL, border=5)

        vbox = wx.BoxSizer(wx.VERTICAL)
        self.done_button = wx.Button(container, label=T("End"))
        self.done_button.Hide()
        self.done_button.Bind(wx.EVT_BUTTON, self.on_done)
        self.done_button.SetBackgroundColour(wx.Colour(255, 230, 230)) 
        self.send_button = wx.Button(container, label=T("Send"))
        self.send_button.Bind(wx.EVT_BUTTON, self.on_send)
        vbox.Add(self.done_button, 0, wx.ALIGN_CENTER | wx.LEFT | wx.RIGHT | wx.BOTTOM, 10)
        vbox.AddSpacer(10)
        vbox.Add(self.send_button, 0, wx.ALIGN_CENTER | wx.LEFT | wx.RIGHT | wx.TOP, 10)

        hbox.Add(vbox, 0, wx.ALIGN_CENTER)
        container.SetSizer(hbox)    
        self.container = container
        return container
    
    def make_panel(self):
        panel = wx.Panel(self)
        vbox = wx.BoxSizer(wx.VERTICAL)

        self.webview = wx.html2.WebView.New(panel)
        self.webview.LoadURL(f'file://{self.html_file_path}')
        self.webview.SetWindowStyleFlag(wx.BORDER_NONE)
        vbox.Add(self.webview, proportion=1, flag=wx.EXPAND | wx.ALL, border=12)

        if sys.platform == 'darwin':
            input_panel = self.make_input_panel(panel)
        else:
            input_panel = self.make_input_panel2(panel)
        drop_target = FileDropTarget(self.input)
        self.input.SetDropTarget(drop_target)
        font = wx.Font(16, wx.FONTFAMILY_DEFAULT, wx.FONTSTYLE_NORMAL, wx.FONTWEIGHT_NORMAL)
        self.input.SetFont(font)
        self.input.SetFocus()
    
        vbox.Add(input_panel, proportion=0, flag=wx.EXPAND | wx.LEFT | wx.RIGHT, border=12)

        panel.SetSizer(vbox)
        self.panel = panel

    def make_menu_bar(self):
        menubar = wx.MenuBar()
        
        # 文件菜单
        file_menu = wx.Menu()
        save_item = file_menu.Append(wx.ID_SAVE, T("Save chat history as HTML"))
        clear_item = file_menu.Append(wx.ID_CLEAR, T("Clear chat"))
        file_menu.AppendSeparator()
        exit_item = file_menu.Append(wx.ID_EXIT, T("Exit"))
        menubar.Append(file_menu, T("File"))
        
        # 编辑菜单
        edit_menu = wx.Menu()
        config_item = edit_menu.Append(wx.ID_ANY, T("Configuration"))
        api_market_item = edit_menu.Append(wx.ID_ANY, T("API Market"))
        menubar.Append(edit_menu, T("Edit"))
        
        # 任务菜单
        task_menu = wx.Menu()
        self.new_task_item = task_menu.Append(wx.ID_NEW, T("Start new task"))
        self.stop_task_item = task_menu.Append(wx.ID_ANY, T("Stop task"))
        self.stop_task_item.Enable(False)
        self.Bind(wx.EVT_MENU, self.on_stop_task, self.stop_task_item)
        self.share_task_item = task_menu.Append(wx.ID_ANY, T("Share task"))
        self.share_task_item.Enable(False)
        self.Bind(wx.EVT_MENU, self.on_share_task, self.share_task_item)
        menubar.Append(task_menu, T("Task"))

        # 帮助菜单
        help_menu = wx.Menu()
        website_item = help_menu.Append(wx.ID_ANY, T("Website"))
        #forum_item = help_menu.Append(wx.ID_ANY, T('Forum'))
        if get_lang() == 'zh':
            wechat_item = help_menu.Append(wx.ID_ANY, T("WeChat Group"))
            self.Bind(wx.EVT_MENU, self.on_open_website, wechat_item)
        help_menu.AppendSeparator()
        about_item = help_menu.Append(wx.ID_ABOUT, T("About"))
        menubar.Append(help_menu, T("Help"))
        
        self.SetMenuBar(menubar)
        
        # 绑定事件
        self.Bind(wx.EVT_MENU, self.on_save_html, save_item)
        self.Bind(wx.EVT_MENU, self.on_clear_chat, clear_item)
        self.Bind(wx.EVT_MENU, self.on_exit, exit_item)
        self.Bind(wx.EVT_MENU, self.on_done, self.new_task_item)
        self.Bind(wx.EVT_MENU, self.on_config, config_item)
        self.Bind(wx.EVT_MENU, self.on_api_market, api_market_item)
        self.Bind(wx.EVT_MENU, self.on_open_website, website_item)
        #self.Bind(wx.EVT_MENU, self.on_open_website, forum_item)
        self.Bind(wx.EVT_MENU, self.on_about, about_item)

    def on_exit(self, event):
        self.task_queue.put('exit')
        self.aipython.join()
        self.Close()

    def on_stop_task(self, event):
        self.aipython.stop_task()

    def on_done(self, event):
        self.task_queue.put('/done')
        
    def on_task_done(self):
        self.done_button.Hide()
        self.SetStatusText(T("Current task has ended"), 0)
        self.new_task_item.Enable(False)
        self.SetTitle(self.title)
        self.clear_chat()

    def on_container_resize(self, event):
        container_size = event.GetSize()
        self.input.SetSize(container_size)

        overlap = -20
        send_button_size = self.send_button.GetSize()
        button_pos_x = container_size.width - send_button_size.width + overlap
        button_pos_y = container_size.height - send_button_size.height - 10
        self.send_button.SetPosition((button_pos_x, button_pos_y))

        if self.aipython.can_done():
            done_button_size = self.done_button.GetSize()
            button_pos_x = container_size.width - done_button_size.width + overlap
            button_pos_y = 10
            self.done_button.SetPosition((button_pos_x, button_pos_y))
            self.done_button.Show()

        event.Skip()

    def on_clear_chat(self, event):
        self.webview.LoadURL(f'file://{self.html_file_path}')

    def clear_chat(self):
        """清空聊天记录"""
        js_code = """
        const chatContainer = document.querySelector('.chat-container');
        chatContainer.innerHTML = '';
        lastUser = '';
        lastMessage = null;
        lastRawContent = '';
        """
        self.webview.RunScript(js_code)

    def on_open_website(self, event):
        """打开网站"""
        menu_item = self.GetMenuBar().FindItemById(event.GetId())
        if not menu_item:
            return
            
        label = menu_item.GetItemLabel()
        if label == T("Website"):
            url = T("https://aipy.app")
        elif label == T("Forum"):
            url = T("https://d.aipy.app")
        elif label == T("WeChat Group"):
            url = T("https://d.aipy.app/d/13")
        else:
            return
        wx.LaunchDefaultBrowser(url)

    def on_about(self, event):
        about_dialog = AboutDialog(self)
        about_dialog.ShowModal()
        about_dialog.Destroy()

    def on_save_html(self, event):
        try:
            html_content = self.webview.GetPageSource()
            self.save_html_content(html_content)
        except Exception as e:
            wx.MessageBox(f"save html error: {e}", "Error")

    def save_html_content(self, html_content):
        with FileDialog(self, T("Save chat history as HTML file"), wildcard="HTML file (*.html)|*.html",
                        style=FD_SAVE | FD_OVERWRITE_PROMPT) as dialog:
            if dialog.ShowModal() == wx.ID_CANCEL:
                return

            path = dialog.GetPath()
            try:
                with open(path, 'w', encoding='utf-8') as file:
                    file.write(html_content)
            except IOError:
                wx.LogError(f"{T('Failed to save file')}: {path}")

    def on_key_down(self, event):
        keycode = event.GetKeyCode()
        send_shortcut = (event.ControlDown() or event.CmdDown()) and keycode == wx.WXK_RETURN

        if send_shortcut:
            self.send_message()
        else:
            event.Skip()

    def on_send(self, event):
        self.send_message()

    def get_task(self):
        return self.task_queue.get()

    def toggle_input(self):
        if self.container.IsShown():
            self.container.Hide()
            self.done_button.Hide()
            wx.BeginBusyCursor()
            self.SetStatusText(T("Operation in progress, please wait..."), 0)
            self.new_task_item.Enable(False)
            self.stop_task_item.Enable(True)
            self.share_task_item.Enable(False)
        else:
            self.container.Show()
            self.done_button.Show()
            wx.EndBusyCursor()
            self.SetStatusText(T("Operation completed. If you start a new task, please click the `End` button"), 0)
            self.new_task_item.Enable(self.aipython.can_done())
            self.stop_task_item.Enable(False)
            self.share_task_item.Enable(True)

        self.panel.Layout()
        self.panel.Refresh()

    def send_message(self):
        text = self.input.GetValue().strip()
        if not text:
            return
        
        if not self.aipython.has_task():
            self.SetTitle(f"[{T('Current task')}] {text}")

        self.append_message(T("Me"), text)
        self.input.Clear()
        self.toggle_input()
        self.task_queue.put(text)

    def on_chat(self, event):
        user = event.user
        text = event.msg
        self.append_message(user, text)

    def append_message(self, user, text):
        avatar = self.avatars[user]
        js_code = f'appendMessage("{avatar}", "{user}", {repr(text)});'
        self.webview.RunScript(js_code)

    def on_config(self, event):
        dialog = ConfigDialog(self, self.tm.settings)
        if dialog.ShowModal() == wx.ID_OK:
            values = dialog.get_values()
            if values['timeout'] == 0:
                del values['timeout']
            self.tm.config_manager.update_sys_config(values)
        dialog.Destroy()

    def on_api_market(self, event):
        """打开API配置对话框"""
        dialog = ApiMarketDialog(self, self.tm.config_manager)
        dialog.ShowModal()
        dialog.Destroy()

    def on_llm_config(self, event):
        """打开LLM配置向导"""
        show_provider_config(self.tm.llm_config, parent=self)

    def on_webview_title_changed(self, event):
        """WebView 标题改变时的处理"""
        if not self.welcomed:
            wx.CallLater(100, self.append_message, T("AIPy"), T("""Hello! I am **AIPy**, your intelligent task assistant!
Please allow me to introduce the other members of the team:
- Turing: The strongest artificial intelligence, complex task analysis and planning
- BB-8: The strongest robot, responsible for executing tasks

Note: Click the "**Help**" link in the menu bar to contact the **AIPy** official and join the group chat."""))
            self.welcomed = True

            # 检查更新
            try:
                update = self.tm.get_update()
                if update and update.get('has_update'):
                    wx.CallLater(1000, self.append_message, T("AIPy"), f"\n🔔 **{T('Update available')}❗**: `v{update.get('latest_version')}`")
            except Exception as e:
                self.log.error(f"检查更新时出错: {e}")
            
        event.Skip()

    def on_share_task(self, event):
        """分享当前任务记录"""
        try:
            html_content = self.webview.GetPageSource()
            result = self.tm.diagnose.report_data(html_content, 'task_record.html')
            self.log.info(f"分享任务记录: {result}")
            if result.get('success'):
                dialog = ShareResultDialog(self, result['url'])
                dialog.ShowModal()
                dialog.Destroy()
            else:
                dialog = ShareResultDialog(self, None, result.get('error'))
                dialog.ShowModal()
                dialog.Destroy()
        except Exception as e:
            dialog = ShareResultDialog(self, None, str(e))
            dialog.ShowModal()
            dialog.Destroy()

class ShareResultDialog(wx.Dialog):
    def __init__(self, parent, url, error=None):
        super().__init__(parent, title=T("Share result"), size=(400, 200))
        logger.info(f"ShareResultDialog: {url}, {error}")
        vbox = wx.BoxSizer(wx.VERTICAL)
        
        if error:
            # 显示错误信息
            error_text = wx.StaticText(self, label=T("Share failed"))
            error_text.SetForegroundColour(wx.Colour(255, 0, 0))
            vbox.Add(error_text, 0, wx.ALL | wx.ALIGN_CENTER, 10)
            
            error_msg = wx.StaticText(self, label=error)
            error_msg.Wrap(350)
            vbox.Add(error_msg, 0, wx.ALL | wx.ALIGN_CENTER, 10)
        else:
            # 显示成功信息
            success_text = wx.StaticText(self, label=T("Share success"))
            success_text.SetForegroundColour(wx.Colour(0, 128, 0))
            vbox.Add(success_text, 0, wx.ALL | wx.ALIGN_CENTER, 10)
            
            # 添加提示文本
            hint_text = wx.StaticText(self, label=T("Click the link below to view the task record"))
            vbox.Add(hint_text, 0, wx.ALL | wx.ALIGN_CENTER, 5)
            
            # 添加可点击的链接
            link = HyperLinkCtrl(self, -1, T("View task record"), URL=url)
            link.EnableRollover(True)
            link.SetUnderlines(False, False, True)
            vbox.Add(link, 0, wx.ALL | wx.ALIGN_CENTER, 5)
        
        # 添加确定按钮
        ok_button = wx.Button(self, wx.ID_OK, T("OK"))
        vbox.Add(ok_button, 0, wx.ALL | wx.ALIGN_CENTER, 10)
        
        self.SetSizer(vbox)
        self.Centre()


def main(args):
    app = wx.App(False)
    conf = ConfigManager(args.config_dir)
    settings = conf.get_config()
    lang = settings.get('lang')
    if lang: set_lang(lang)
    llm_config = LLMConfig(CONFIG_DIR / "config")
    if conf.check_config(gui=True) == 'TrustToken':
        if llm_config.need_config():
            show_provider_config(llm_config)
            if llm_config.need_config():
                return
        settings["llm"] = llm_config.config
        
    settings.gui = True
    settings.debug = args.debug
    settings.auto_install = True
    settings.auto_getenv = True

    file = None if args.debug else open(os.devnull, 'w', encoding='utf-8')
    console = Console(file=file, record=True)
    console.gui = True
    try:
        tm = TaskManager(settings, console=console, gui=True)
    except Exception as e:
        traceback.print_exc()
        return
    tm.config_manager = conf
    tm.llm_config = llm_config
    ChatFrame(tm)
    app.MainLoop()



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import json
from typing import Dict
from pathlib import Path

class BaseConfig:
    FILE = None

    def __init__(self, path: str):
        path = Path(path)
        path.mkdir(parents=True, exist_ok=True)
        self.config_file = path / self.FILE
        self.config = self.load_config()

    def load_config(self) -> Dict:
        if self.config_file.exists():
            with open(self.config_file, 'r') as f:
                return json.load(f)
        return {}

    def save_config(self, config: Dict):
        with open(self.config_file, 'w') as f:
            json.dump(config, f, indent=2)


#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import json
import requests

from . import BaseClient, ChatMessage

# https://github.com/ollama/ollama/blob/main/docs/api.md
class OllamaClient(BaseClient):
    def __init__(self, config):
        super().__init__(config)
        self._session = requests.Session()

    def usable(self):
        return super().usable() and self._base_url
    
    def _parse_usage(self, response):
        ret = {'input_tokens': response['prompt_eval_count'], 'output_tokens': response['eval_count']}
        ret['total_tokens'] = ret['input_tokens'] + ret['output_tokens']
        return ret

    def _parse_stream_response(self, response, stream_processor):
        with stream_processor as lm:
            for chunk in response.iter_lines():
                chunk = chunk.decode(encoding='utf-8')
                msg = json.loads(chunk)
                if msg['done']:
                    usage = self._parse_usage(msg)
                    break

                if 'message' in msg and 'content' in msg['message'] and msg['message']['content']:
                    content = msg['message']['content']
                    lm.process_chunk(content)

        return ChatMessage(role="assistant", content=lm.content, usage=usage)

    def _parse_response(self, response):
        response = response.json()
        msg = response["message"]
        return ChatMessage(role=msg['role'], content=msg['content'], usage=self._parse_usage(response))
    
    def get_completion(self, messages):
        response = self._session.post(
            f"{self._base_url}/api/chat",
            json={
                "model": self._model,
                "messages": messages,
                "stream": self._stream,
                "options": {"num_predict": self.max_tokens}
            },
            timeout=self._timeout,
            **self._params
        )
        response.raise_for_status()
        return response



import sys
import re
import io
import datetime
from pathlib import Path
import traceback

from dynaconf import Dynaconf
from rich import print
import tomli_w

from .. import __respath__, T
from .trustoken import TrustToken
from .libmcp import MCPToolManager

__PACKAGE_NAME__ = "aipyapp"

OLD_SETTINGS_FILES = [
    Path.home() / '.aipy.toml',
    Path('aipython.toml').resolve(),
    Path('.aipy.toml').resolve(),
    Path('aipy.toml').resolve(),
]

CONFIG_FILE_NAME = f"{__PACKAGE_NAME__}.toml"
USER_CONFIG_FILE_NAME = "user_config.toml"
MCP_CONFIG_FILE_NAME = "mcp.json"

def get_tt_aio_api(tt_api_key) -> dict:
    """
    获取 TrustToken AIO API Key
    :param tt_api_key: API Key

    """
    if not tt_api_key:
        return {}

    search_url = f"{T('https://sapi.trustoken.ai/aio-api')}/search/unified"
    geoip_url = f"{T('https://sapi.trustoken.ai/aio-api')}/ipgeo"
    amap_url = f"{T('https://sapi.trustoken.ai/aio-api')}/amap"
    tt_aio_api = {
        'tt_aio_map': {
            'env': {'tt_aio_map': [tt_api_key, "最新地图API Key"]},
            'desc': f"""高德地图（地理编码、驾车、骑行、步行、公交路线规划，周边关键字搜索，天气查询，交通态势、店铺查询, 无法确定POI分类编码时请用关键字搜索API)，**参数中的origin、destination都是坐标**
当需要访问`https://restapi.amap.com/`时，请使用`{amap_url}/`代替，两者API接口完全一致""",
        },
        'tt_aio_geoip':{
            'env': {'tt_aio_geoip': [tt_api_key, "最新IP地理位置API Key"]},
            'desc': f"""如果任务中涉及到位置，但没有指定具体位置，可以用此接口获取地理位置信息，包括国家、省份、城市等信息。接口调用示例如下：
curl -H 'Authorization: Bearer xxx' {geoip_url}
响应数据如下：{{"city": "成都", "country": "中国", "ip": "171.2.1.1", "isp": "电信", "latitude": "32.676235", "longitude": "103.058986", "province": "四川", "version": 4}}""",
        },
        'tt_aio_search': {
            'env': {'tt_aio_search': [tt_api_key, "Trustoken网络搜索API Key"]},
            'desc': f"""联网搜索服务，用于搜索网络信息, **注意：1. 用户指定了搜索引擎时，请勿使用此API；2. 不支持指定时间、网站搜索**。仅在必须联网搜索时调用，接口调用示例如下：
curl  -X POST {search_url} \
--header "Authorization: Bearer xxxxx" \
--header "Content-Type: application/json" \
--data '{{"query": "网络搜索内容", "contents":{{"markdownText":true}}}}'
接口返回的数据样例：
{{
"pageItems": [{{
    "title": "网页标题",
    "link": "https://...",
    "snippet": "网页摘要",
    "publishedTime": "2025-03-09T22:13:38+08:00",
    "markdownText": "网页内容",
    "images": [
        "图片地址"
    ],
    "hostname": "网站名",
}}]
}}""",
        },
    }

    return tt_aio_api


def init_config_dir():
    """
    获取平台相关的配置目录，并确保目录存在
    """
    config_dir = Path.home() / f".{__PACKAGE_NAME__}"
    # 确保目录存在
    try:
        config_dir.mkdir(parents=True, exist_ok=True)
    except PermissionError:
        print(T("Permission denied to create directory: {}").format(config_dir))
        raise
    except Exception as e:
        print(T("Error creating configuration directory: {}").format(config_dir, str(e)))
        raise

    return config_dir


CONFIG_DIR = init_config_dir()
PLUGINS_DIR = CONFIG_DIR / "plugins"
TIPS_DIR = CONFIG_DIR / "tips"

def get_config_file_path(config_dir=None, file_name=CONFIG_FILE_NAME, create=True):
    """
    获取配置文件的完整路径
    :return: 配置文件的完整路径
    """
    if config_dir:
        config_dir = Path(config_dir)
    else:
        config_dir = CONFIG_DIR

    config_file_path = config_dir / file_name

    # 如果配置文件不存在，则创建一个空文件
    if not config_file_path.exists() and create:
        try:
            config_file_path.touch()
        except Exception as e:
            print(T("Error creating configuration directory: {}").format(config_file_path))
            raise

    return config_file_path


def lowercase_keys(d):
    """递归地将字典中的所有键转换为小写"""
    if not isinstance(d, dict):
        return d
    return {k.lower(): lowercase_keys(v) for k, v in d.items()}


def is_valid_api_key(api_key):
    """
    校验是否为有效的 API Key 格式。
    API Key 格式为字母、数字、减号、下划线的组合，长度在 8 到 128 之间
    :param api_key: 待校验的 API Key 字符串
    :return: 如果格式有效返回 True，否则返回 False
    """
    pattern = r"^[A-Za-z0-9_-]{8,128}$"
    return bool(re.match(pattern, api_key))


def get_mcp(config_dir=None):
    mcp_config_file = get_config_file_path(
        config_dir, MCP_CONFIG_FILE_NAME, create=False
    )
    # exists and not empty
    if not mcp_config_file.exists() or mcp_config_file.stat().st_size == 0:
        return None
    return MCPToolManager(mcp_config_file)


def get_tt_api_key(settings=None) -> str:
    """获取 TrustToken API Key
    :param settings: 配置对象
    :return: API Key 字符串
    """
    if not settings or not isinstance(settings, Dynaconf):
        return ""

    key = settings.get('llm', {}).get('Trustoken', {}).get('api_key')
    if not key:
        key = settings.get('llm', {}).get('trustoken', {}).get('api_key')
    if not key:
        return ""
    return key


class ConfigManager:
    def __init__(self, config_dir=None):
        self.config_file = get_config_file_path(config_dir)
        self.user_config_file = get_config_file_path(config_dir, USER_CONFIG_FILE_NAME)
        self.default_config = __respath__ / "default.toml"
        self.config = self._load_config()

        self.config.update({'_config_dir': config_dir})

        self.trust_token = TrustToken()
        # print(self.config.to_dict())

    def get_work_dir(self):
        if self.config.workdir:
            return Path.cwd() / self.config.workdir
        return Path.cwd()

    def _load_config(self, settings_files=[]):
        """加载配置文件
        :param settings_files: 配置文件列表
        :return: 配置对象
        """
        if not settings_files:
            # 新版本配置文件
            settings_files = [
                self.default_config,
                self.user_config_file,
                self.config_file,
            ]
        # 读取配置文件
        try:
            config = Dynaconf(
                settings_files=settings_files, envvar_prefix="AIPY", merge_enabled=True
            )

            # check if it's a valid config
            assert config.to_dict()
        except Exception as e:
            # 配置加载异常处理
            # print(T('error_loading_config'), str(e))
            # 回退到一个空配置实例，避免后续代码因 config 未定义而出错
            config = Dynaconf(
                settings_files=[], envvar_prefix="AIPY", merge_enabled=True
            )
        return config

    def reload_config(self):
        self.config = self._load_config()
        return self.config

    def get_config(self):
        return self.config

    def update_sys_config(self, new_config, overwrite=False):
        """更新aipyapp.toml配置文件
        :param new_config: 新配置字典, 如 {"workdir": "/path/to/workdir"}
        """
        # 加载系统配置文件
        assert isinstance(new_config, dict)

        if overwrite:
            # 如果需要覆盖，则直接使用新的配置
            config = Dynaconf(
                settings_files=[], envvar_prefix="AIPY", merge_enabled=True
            )
        else:
            config = self._load_config(settings_files=[self.config_file])

        config.update(new_config)

        # 保存到配置文件
        header_comments = [
            f"# Configuration file for {__PACKAGE_NAME__}",
            "# Auto-generated on "
            + datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S"),
            f"# 请勿直接修改此文件，除非您了解具体配置格式，如果自定义配置，请放到{self.user_config_file}",
            f"# Please do not edit this file directly unless you understand the format. If you want to customize the configuration, please edit {self.user_config_file}",
            "",
        ]
        footer_comments = ["", "# End of configuration file"]
        # print(config.to_dict())
        cfg_dict = lowercase_keys(config.to_dict())

        with open(self.config_file, "w", encoding="utf-8") as f:
            # 1. 写入头部注释
            f.write("\n".join(header_comments) + "\n")

            # 2. 写入 TOML 内容到临时内存文件
            temp_buffer = io.BytesIO()
            tomli_w.dump(cfg_dict, temp_buffer, multiline_strings=True)
            toml_content = temp_buffer.getvalue().decode('utf-8')

            # 3. 写入 TOML 内容
            f.write(toml_content)

            # 4. 写入尾部注释
            f.write("\n".join(footer_comments))
        return config

    def update_api_config(self, new_api_config):
        """更新配置文件中的API定义"""
        assert isinstance(new_api_config, dict)
        assert 'api' in new_api_config
        config = self._load_config(settings_files=[self.config_file])
        cfg_dict = lowercase_keys(config.to_dict())
        cfg_dict.pop('api', None)
        cfg_dict.update(new_api_config)

        # 配置overwrite
        self.update_sys_config(cfg_dict, overwrite=True)

    def save_tt_config(self, api_key):
        config = {
            'llm': {
                'trustoken': {
                    'api_key': api_key,
                    'type': 'trust',
                    'base_url': T("https://sapi.trustoken.ai/v1"),
                    'model': 'auto',
                    'default': True,
                    'enable': True,
                }
            }
        }
        self.update_sys_config(config)
        return config

    def check_llm(self):
        """检查是否有可用的LLM配置。
        只要有可用的配置，就不强制要求trustoken配置。
        """
        llm = self.config.get("llm")
        if not llm:
            print(T("Missing 'llm' configuration."))

        llms = {}
        for name, config in self.config.get('llm', {}).items():
            if config.get("enable", True):
                llms[name] = config

        return llms

    def fetch_config(self):
        """从tt获取配置并保存到配置文件中。"""
        self.trust_token.fetch_token(self.save_tt_config)

    def check_config(self, gui=False):
        """检查配置文件是否存在，并加载配置。
        如果配置文件不存在，则创建一个新的配置文件。
        """
        try:
            if not self.config:
                print(T("Configuration not loaded."))
                return

            if self.check_llm():
                # 有状态为 enable 的配置文件，则不需要强制要求 trustoken 配置。
                return

            # 尝试从旧版本配置迁移
            if self._migrate_config():
                # 迁移完成后重新加载配置
                self.reload_config()

            # 如果仍然没有可用的 LLM 配置，则从网络拉取
            if not self.check_llm():
                if gui:
                    return 'TrustToken'
                self.fetch_config()
                self.reload_config()

            if not self.check_llm():
                print(T("Missing 'llm' configuration."))
                sys.exit(1)

        except Exception as e:
            traceback.print_exc()
            sys.exit(1)

    def _migrate_config(self):
        """
        Migrates configuration from old settings files (OLD_SETTINGS_FILES)
        to the new user_config.toml and potentially creates the main aipyapp.toml
        if a TrustToken configuration is found.
        """
        combined_toml_content = ""
        migrated_files = []
        backup_files = []

        for path in OLD_SETTINGS_FILES:
            if not path.exists():
                continue

            try:
                config = Dynaconf(settings_files=[path])
                config_dict = config.to_dict()
                assert config_dict

                try:
                    content = path.read_text(encoding='utf-8')
                except UnicodeDecodeError:
                    try:
                        content = path.read_text(encoding='gbk')
                    except Exception as e:
                        print(f"Error reading file {path}: {e}")
                        continue

                combined_toml_content += content + "\n\n"  # Add separator

                # 文件内容、格式都正常，则准备迁移
                migrated_files.append(str(path))
                # Backup the old file
                backup_path = path.with_name(f"{path.stem}-backup{path.suffix}")
                try:
                    path.rename(backup_path)
                    backup_files.append(str(backup_path))
                except Exception as e:
                    pass

            except Exception as e:
                pass
        if not combined_toml_content:
            return

        print(T("""Found old configuration files: {}
Attempting to migrate configuration from these files...
After migration, these files will be backed up to {}, please check them.""").format(
                ', '.join(migrated_files), ', '.join(backup_files)
            )
        )

        # Write combined content to user_config.toml
        try:
            with open(self.user_config_file, "w", encoding="utf-8") as f:
                f.write(f"# Migrated from: {', '.join(migrated_files)}\n")
                f.write(f"# Original files backed up to: {', '.join(backup_files)}\n\n")
                f.write(combined_toml_content)
                print(T("Successfully migrated old version user configuration to {}").format(self.user_config_file))
        except Exception as e:
            return

        # Now, load the newly created user config to find TT key
        try:
            temp_config = self._load_config(settings_files=[self.user_config_file])
            llm_config = temp_config.get('llm', {})

            for section_name, section_data in llm_config.items():
                if isinstance(section_data, dict) and self._is_tt_config(
                    section_name, section_data
                ):
                    api_key = section_data.get('api_key', section_data.get('api-key'))
                    if api_key:
                        # print("Token found:", api_key)
                        self.save_tt_config(api_key)
                        print(T("Successfully migrated old version trustoken configuration to {}").format(self.config_file))
                        break

        except Exception as e:
            pass

        return True

    def _is_tt_config(self, name, config):
        """
        判断配置是否符合特定条件

        参数:
            name: 配置名称
            config: 配置内容字典

        返回: 如果符合条件返回True
        """
        # 条件1: 配置名称包含目标关键字
        if any(keyword in name.lower() for keyword in ['trustoken', 'trust']):
            return True

        base_url = config.get('base_url', config.get('base-url', '')).lower()
        # 条件2: base_url包含目标域名
        if isinstance(config, dict) and base_url:
            if 'trustoken.' in base_url:
                return True

        # 条件3: 其他特定标记
        # type == trust, 且没有base_url.
        if isinstance(config, dict) and config.get('type') == 'trust' and not base_url:
            return True

        return False



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import threading
import traceback
import importlib.util
from typing import Callable, Any, Dict, List

class EventBus:
    def __init__(self):
        self._listeners: Dict[str, List[Callable[..., Any]]] = {}
        self._stop_event = threading.Event()

    def stop(self):
        self._stop_event.set()

    def is_stopped(self):
        return self._stop_event.is_set()    

    def __repr__(self):
        return repr(self._listeners)
    
    def register(self, event_name: str, handler: Callable[..., Any]):
        self._listeners.setdefault(event_name, []).append(handler)

    def broadcast(self, event_name: str, *args, **kwargs):
        for handler in self._listeners.get(event_name, []):
            try:
                handler(*args, **kwargs)
            except Exception as e:
                traceback.print_exc()

    def pipeline(self, event_name: str, data, **kwargs):
        for handler in self._listeners.get(event_name, []):
            try:
                handler(data, **kwargs)
            except Exception as e:
                traceback.print_exc()

    def collect(self, event_name: str, *args, **kwargs):
        try:
            ret = [handler(*args, **kwargs) for handler in self._listeners.get(event_name, [])]
        except Exception as e:
            ret = []
            traceback.print_exc()
        return ret

    def __call__(self, event_name: str, *args, **kwargs):
        self.pipeline(event_name, *args, **kwargs)

event_bus = EventBus()

class PluginManager:
    def __init__(self, plugin_dir: str):
        # Get the system plugin directory
        # This is the directory where the `aio_api.py` file is located
        self.sys_plugin_dir = os.path.join(os.path.dirname(os.path.dirname(__file__)), 'plugins')
        self.plugin_dir = plugin_dir
        self.plugins: Dict[str, Any] = {}

    def load_plugins(self):
        """Load plugins from the plugin directory."""
        for plugin_dir in [self.sys_plugin_dir, self.plugin_dir]:
            if not os.path.exists(plugin_dir):
                continue
            for fname in os.listdir(plugin_dir):
                if fname.endswith(".py") and not fname.startswith("_"):
                    self._load_plugin(os.path.join(plugin_dir, fname))


    def _load_plugin(self, filepath: str):
        plugin_id = os.path.basename(filepath)[:-3]

        spec = importlib.util.spec_from_file_location(plugin_id, filepath)
        module = importlib.util.module_from_spec(spec)
        spec.loader.exec_module(module)

        plugin_cls = getattr(module, "Plugin", None)
        if not plugin_cls or not callable(plugin_cls):
            return

        plugin = plugin_cls()

        for attr_name in dir(plugin):
            if attr_name.startswith("on_") and len(attr_name) > 3:
                handler = getattr(plugin, attr_name)
                if callable(handler):
                    event_bus.register(attr_name[3:], handler)

        self.plugins[plugin_id] = plugin



import shlex
import argparse
from collections import OrderedDict

from ... import T
from .base import BaseCommand
from .cmd_info import InfoCommand
from .cmd_help import HelpCommand
from .cmd_llm import LLMCommand
from .cmd_use import UseCommand
from .cmd_env import EnvCommand
from .cmd_task import TaskCommand
from .cmd_mcp import MCPCommand

from loguru import logger
from rich import print
from prompt_toolkit.completion import Completer, Completion, NestedCompleter
from prompt_toolkit.key_binding import KeyBindings

COMMANDS = [InfoCommand, UseCommand, EnvCommand, LLMCommand, TaskCommand, MCPCommand, HelpCommand]

class CommandManager(Completer):
    def __init__(self, tm):
        self.tm = tm
        self.commands = OrderedDict()
        self.log = logger.bind(src="CommandManager")
        self.init()

    def init(self):
        """Initialize all registered commands"""
        for command in COMMANDS:
            self.register_command(command())
        
        for command in self.commands.values():
            command.init()

    def register_command(self, command_instance):
        """Register a command instance"""
        if not isinstance(command_instance, BaseCommand):
            raise ValueError("Command must be an instance of BaseCommand")
        if command_instance.name in self.commands:
            raise ValueError(f"Command '{command_instance.name}' is already registered")
        self.commands[command_instance.name] = command_instance
        command_instance.manager = self

    def get_completions(self, document, complete_event):
        text = document.text_before_cursor
        if not text.startswith('/'):
            return

        text = text[1:]
        try:
            words = shlex.split(text)
        except ValueError as e:
            self.log.error(f"输入解析错误: {e}")
            return
        
        # 提取公共补全函数
        def complete_items(items, partial, start_pos=None):
            start_pos = -len(partial) if start_pos is None else start_pos
            for item in items:
                if item.name.startswith(partial):
                    yield Completion(
                        item.name,
                        start_position=start_pos,
                        display_meta=item.desc
                    )

        if len(words) == 0 or (len(words) == 1 and not text.endswith(' ')):
            # 补全主命令
            partial_cmd = words[0] if len(words) > 0 else ''
            yield from complete_items(self.commands.values(), partial_cmd)
            return

        cmd = words[0]
        if cmd not in self.commands:
            return

        command_instance = self.commands[cmd]
        subcommands = command_instance.subcommands

        # 补全子命令
        if subcommands:
            if len(words) == 1 or (len(words) == 2 and not text.endswith(' ')):
                partial_subcmd = words[1] if len(words) > 1 else ''
                yield from complete_items(subcommands.values(), partial_subcmd)
                return

            subcmd = words[1]
            if subcmd not in subcommands:
                return
            arguments = subcommands[subcmd]['arguments']
        else:
            subcmd = None
            arguments = command_instance.arguments

        if text.endswith(' '):
            partial_arg = ''
            last_word = words[-1]
        else:
            partial_arg = words[-1]
            last_word = words[-2]

        arg = arguments.get(last_word, None)
        if arg and arg['requires_value']:
            choices = command_instance.get_arg_values(arg, subcmd)
            if choices:
                yield from complete_items(choices, partial_arg)
            return

        yield from complete_items(arguments.values(), partial_arg)

    def execute(self, user_input):
        """Execute a command"""
        if not user_input.startswith('/'):
            return
        
        user_input = user_input[1:].strip()
        if not user_input:
            return
        
        args = shlex.split(user_input)
        command = args[0]
        if command not in self.commands:
            print(f"Unknown command: {command}")
            return
        
        command_instance = self.commands[command]
        parser = command_instance.parser
        
        try:
            # Parse remaining arguments (excluding the command name)
            parsed_args = parser.parse_args(args[1:])
            parsed_args.raw_args = args[1:]
            command_instance.execute(parsed_args)
        except SystemExit:
            pass
        except argparse.ArgumentError as e:
            print(f"Argument error: {e}")
        except Exception as e:
            print(f"Error: {e}")

class TaskCommandManager(NestedCompleter):
    def __init__(self, tm):
        names = tm.client_manager.names['enabled']
        commands = {
            '/use': NestedCompleter.from_nested_dict(dict.fromkeys(names)),
            '/done': None,
        }
        commands['use'] = commands['/use']
        commands['done'] = commands['/done']
        for name in names:
            commands[name] = None
        super().__init__(commands)
        self.meta_dict = {
            'use': T('Switch LLM'),
            'done': T('End'),
        }
        self.names = names
        self.log = logger.bind(src="TaskCommandManager")

    def get_completions(self, document, complete_event):
        completions = super().get_completions(document, complete_event)
        for completion in completions:
            text = completion.text
            if text.startswith('/'):
                text = text[1:]
            completion._display_meta = self.meta_dict.get(text, '')
            yield completion
            
    def execute(self, task, user_input):
        """Execute a command
        return True if the command is processed, otherwise False
        """
        words = user_input.split()
        if len(words) == 1 and words[0] in self.names:
            task.use(words[0])
            return True
        elif words[0] in ('use', '/use'):
            if len(words) == 2 and words[1] in self.names:
                task.use(words[1])
            else:
                print(f"Unknown client: {words[1]}")
            return True
        return False


from rich import print
from rich.table import Table
import random

COLORS = ["red", "green", "blue", "yellow", "magenta", "cyan", "white", "bright_red", "bright_green", "bright_blue", "bright_yellow", "bright_magenta", "bright_cyan"]

def print_table(rows, title=None, headers=None):
    if not rows:
        return
    table = Table(title=title, show_lines=True)
    for header in headers:  
        # 为每一列随机选择一个颜色
        color = random.choice(COLORS)
        table.add_column(header, justify="center", style=f"bold {color}", no_wrap=True)
    for row in rows:
        table.add_row(*[str(cell) for cell in row])
    print(table)

def print_records(records, title=None):
    if not records:
        return
    headers = type(records[0])._fields
    print_table(records, title=title, headers=headers)



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import sys
import json
import uuid
import time
import platform
import locale
import mimetypes
from io import BytesIO

import requests
from loguru import logger

from aipyapp import __version__
from aipyapp.aipy.config import CONFIG_DIR

CONFIG_FILE = CONFIG_DIR / '.diagnose.json'
UPDATE_INTERVAL = 8 * 3600

class NoopDiagnose:
    def __getattr__(self, name):
        def noop(*args, **kwargs):
            pass
        return noop

class Diagnose:
    def __init__(self, api_url, api_key):
        self._api_url = api_url
        self._api_key = api_key
        self.log = logger.bind(src='diagnose')
        self.load_config()

    def save_config(self):
        CONFIG_FILE.write_text(json.dumps({'xid': self._xid, 'last_update': self._last_update}))

    def load_config(self):
        if not CONFIG_FILE.exists():
            self._xid = uuid.uuid4().hex
            self._last_update = 0
            self.save_config()
        else:
            with CONFIG_FILE.open('r') as f:
                data = json.load(f)
                self._xid = data.get('xid')
                self._last_update = data.get('last_update')

    @classmethod
    def create(cls, settings):
        config = settings.get('diagnose')
        if config:
            api_key = config.get('api_key')
            api_url = config.get('api_url')
            enabled = config.get('enabled', True)
            if not api_url:
                enabled = False
        else:
            enabled = False

        return cls(api_url, api_key) if enabled else NoopDiagnose()

    def get_meta(self):
        return {
            "system": platform.system(),
            "version": platform.version(),
            "platform": platform.platform(),
            "arch": platform.machine(),
            "python": sys.executable,
            "python_version": sys.version,
            "python_base_prefix": sys.base_prefix,
            "locale": locale.getlocale(),
        }
    
    def check_update(self, force=False):
        if not force and int(time.time()) - self._last_update < UPDATE_INTERVAL:
            return {}
        self._last_update = int(time.time())
        self.save_config()
        
        data = {
            "xid": self._xid,
            "version": __version__,
            "meta": self.get_meta()
        }
        headers = {
            "Content-Type": "application/json",
            #"API-KEY": self._api_key
        }

        try:
            response = requests.post(
                f"{self._api_url}/4b16535b7e6147f2861508a2ad5f5ce8",
                json=data,
                headers=headers
            )
            if response.status_code == 200:
                result = response.json()
                if result.get("success", False):
                    return {
                        "has_update": result.get("has_update", False),
                        "latest_version": result.get("latest_version", "unknown"),
                        "current_version": __version__,
                    }
                else:
                    self.log.error(f"Server error: {result.get('error', '未知错误')}")
            else:
                self.log.error(f"Request failed: HTTP {response.status_code}")

        except Exception as e:
            self.log.error(f"Connection error: {str(e)}")
        return {"error": "版本检查失败"}

    def report_data(self, data, filename):
        try:
            # 确保数据是字符串格式
            if isinstance(data, (dict, list)):
                data = json.dumps(data, ensure_ascii=False, indent=4)
            elif not isinstance(data, str):
                data = str(data)
            
            # 创建文件对象
            file_data = BytesIO(data.encode('utf-8'))
            file_data.seek(0)  # 确保文件指针在开始位置
        except Exception as e:
            self.log.error(f"Failed to prepare data: {str(e)}")
            return {'success': False, 'error': str(e)}
        
        headers = {'API-KEY': self._api_key}
        content_type, _ = mimetypes.guess_type(filename)
        if not content_type:
            content_type = 'application/octet-stream'
            
        files = {
            'file': (
                filename,
                file_data,
                content_type
            )
        }

        error = 'Unknown error'
        try:
            response = requests.post(f"{self._api_url}/a6477529c8c34b6a8ca4bc2d7253ab76", files=files, headers=headers)
            if 200 <= response.status_code < 300:
                try:
                    result = response.json()
                    if result.get('success') and 'viewUrl' in result:
                        self.log.info(f"Report uploaded successfully. View URL: {result['viewUrl']}")
                        return {'success': True, 'url': result['viewUrl']}
                    else:
                        self.log.error(f"Upload failed: {result.get('error', 'Unknown error')}")
                except json.JSONDecodeError:
                    error = "Failed to parse response as JSON"
                    self.log.error(error)
            else:
                error = f"Upload failed with status code: {response.status_code}"
                self.log.error(error)
            
        except Exception as e:
            error = f"Failed to upload report: {str(e)}"
            self.log.error(error)
        return {'success': False, 'error': error}

    def report_code_error(self, history):
        # Report code execution errors from history
        # Each history entry contains code and execution result
        # We only collect entries with traceback information
        # Returns True if report was sent successfully
        if not self._api_key:
            return True
        
        data = []
        for h in history:
            result = h.get('result')
            if not result:
                continue
            traceback = result.get('traceback')
            if not traceback:
                continue
            data.append({
                'code': h.get('code'),
                'traceback': traceback,
                'error': result.get('errstr')
            })

        if data:
            return self.report_data(data, 'code_error.json')
        return True

if __name__ == '__main__':
    settings = {
        'diagnose': {
            'api_key': 'sk-aipy-',
            'api_url': 'https://aipy.xxyy.eu.org/',
        }
    }
    diagnose = Diagnose.create(settings)
    update = diagnose.check_update()
    print(update)
    url = diagnose.report_code_error([
        {'code': 'print("Hello, World!")', 'result': {'traceback': 'Traceback (most recent call last):\n  File "test.py", line 1, in <module>\n    print("Hello, World!")\nNameError: name \'print\' is not defined\n', 'errstr': 'NameError: name \'print\' is not defined'}}
    ])
    print(url)




import traceback

import wx
from loguru import logger
from wx.lib.scrolledpanel import ScrolledPanel

from .. import T

class ApiItemPanel(wx.Panel):
    """单个API配置项面板"""
    def __init__(self, parent, api_name, api_config, on_edit=None, on_delete=None):
        super().__init__(parent, style=wx.BORDER_SIMPLE)
        self.api_name = api_name
        self.api_config = api_config
        self.on_edit = on_edit
        self.on_delete = on_delete

        # 创建布局
        main_sizer = wx.BoxSizer(wx.VERTICAL)
        
        # API名称
        name_sizer = wx.BoxSizer(wx.HORIZONTAL)
        name_label = wx.StaticText(self, label=f"{T('API Name')}: {api_name}")
        name_label.SetFont(wx.Font(10, wx.FONTFAMILY_DEFAULT, wx.FONTSTYLE_NORMAL, wx.FONTWEIGHT_BOLD))
        name_sizer.Add(name_label, 1, wx.EXPAND | wx.ALL, 5)
        
        # 添加按钮
        button_sizer = wx.BoxSizer(wx.HORIZONTAL)
        edit_button = wx.Button(self, label=T("Edit"))
        edit_button.Bind(wx.EVT_BUTTON, self.on_edit_click)
        delete_button = wx.Button(self, label=T("Delete"))
        delete_button.Bind(wx.EVT_BUTTON, self.on_delete_click)
        
        button_sizer.Add(edit_button, 0, wx.ALL, 5)
        button_sizer.Add(delete_button, 0, wx.ALL, 5)
        name_sizer.Add(button_sizer, 0, wx.ALIGN_CENTER_VERTICAL | wx.RIGHT, 5)
        
        main_sizer.Add(name_sizer, 0, wx.EXPAND | wx.ALL, 5)
        
        # API详情
        details_sizer = wx.BoxSizer(wx.VERTICAL)
        
        # 显示API KEY(s)
        envs = api_config.get('env', {})
        for key_name in envs:
            key_value = envs[key_name]
            if isinstance(key_value, list) and len(key_value) > 0:
                masked_key = self.mask_api_key(key_value[0])
                display_key = key_name
                key_text = wx.StaticText(self, label=f"{display_key}: {masked_key}")
                details_sizer.Add(key_text, 0, wx.EXPAND | wx.LEFT | wx.RIGHT, 10)
        
        # 显示描述
        if 'desc' in api_config:
            desc_text = wx.StaticText(self, label=f"{T('Description')}: {api_config['desc']}")
            details_sizer.Add(desc_text, 0, wx.EXPAND | wx.LEFT | wx.RIGHT | wx.TOP, 10)
        
        main_sizer.Add(details_sizer, 0, wx.EXPAND | wx.ALL, 5)
        
        self.SetSizer(main_sizer)
        self.Layout()
    
    def mask_api_key(self, key):
        """将API密钥进行掩码处理，只显示前3位和后3位"""
        if not key or len(key) < 8:
            return key
        return key[:3] + "..." + key[-3:]
    
    def on_edit_click(self, event):
        if self.on_edit:
            self.on_edit(self.api_name, self.api_config)
    
    def on_delete_click(self, event):
        if self.on_delete:
            dlg = wx.MessageDialog(
                self, 
                f"{T('Are you sure to delete API')} '{self.api_name}'?",
                T("Confirm Delete"),
                wx.YES_NO | wx.ICON_QUESTION
            )
            result = dlg.ShowModal()
            dlg.Destroy()
            
            if result == wx.ID_YES and self.on_delete:
                self.on_delete(self.api_name)


class ApiEditDialog(wx.Dialog):
    """API编辑对话框"""
    def __init__(self, parent, api_name="", api_config=None, is_new=True):
        title = T("Add API") if is_new else T("Edit API")
        super().__init__(parent, title=title, size=(600, 500))
        
        self.is_new = is_new
        self.api_name = api_name
        self.api_config = api_config or {"desc": ""}
        
        self._init_ui()
        
    def _init_ui(self):
        """初始化UI界面"""
        main_sizer = wx.BoxSizer(wx.VERTICAL)
        
        # API名称
        name_sizer = wx.BoxSizer(wx.HORIZONTAL)
        name_label = wx.StaticText(self, label=T("API Name"))
        self.name_input = wx.TextCtrl(self)
        if not self.is_new:
            self.name_input.SetValue(self.api_name)
            self.name_input.Enable(False)  # 编辑模式下不允许更改名称
        
        name_sizer.Add(name_label, 0, wx.ALL | wx.ALIGN_CENTER_VERTICAL, 5)
        name_sizer.Add(self.name_input, 1, wx.ALL | wx.EXPAND, 5)
        
        main_sizer.Add(name_sizer, 0, wx.EXPAND | wx.ALL, 10)
        
        # API环境变量标题行
        env_header_sizer = wx.BoxSizer(wx.HORIZONTAL)
        env_label = wx.StaticText(self, label=T("API Key Settings"))
        add_env_button = wx.Button(self, label=T("Add Environment Variable"))
        add_env_button.Bind(wx.EVT_BUTTON, self.on_add_env)
        
        env_header_sizer.Add(env_label, 0, wx.ALIGN_CENTER_VERTICAL | wx.RIGHT, 10)
        env_header_sizer.Add(add_env_button, 0, wx.ALIGN_CENTER_VERTICAL)
        main_sizer.Add(env_header_sizer, 0, wx.ALL, 10)
        
         # 环境变量列表
        self.env_panel = wx.ScrolledWindow(self)
        self.env_panel.SetScrollRate(0, 20)
        self.env_sizer = wx.BoxSizer(wx.VERTICAL)
        self.env_panel.SetSizer(self.env_sizer)
        
        # 添加已有的环境变量
        self.env_controls = []
        
        # 查找环境变量键
        env_vars = {}
        if 'env' in self.api_config and isinstance(self.api_config['env'], dict):
            for key, value in self.api_config['env'].items():
                env_vars[key] = value
        
        # 添加找到的环境变量
        for var_name, value in env_vars.items():
            self.add_env_control(var_name, value)
            
        # 如果是新建API且没有环境变量，则默认添加一个空的环境变量控件
        if self.is_new and not env_vars:
            self.add_env_control("api_key", [T("Enter your API key"), T("API key description")])
        
        main_sizer.Add(self.env_panel, 2, wx.EXPAND | wx.ALL, 10)  # 增加比例权重到2
        
        # 描述
        desc_sizer = wx.BoxSizer(wx.VERTICAL)
        desc_label = wx.StaticText(self, label=T("API Description"))
        
        # 使用更大的多行文本框
        self.desc_input = wx.TextCtrl(self, style=wx.TE_MULTILINE)
        # 设置更大的字体和最小高度
        font = wx.Font(10, wx.FONTFAMILY_DEFAULT, wx.FONTSTYLE_NORMAL, wx.FONTWEIGHT_NORMAL)
        self.desc_input.SetFont(font)
        self.desc_input.SetMinSize((-1, 100))  # 设置最小高度
        
        if 'desc' in self.api_config:
            self.desc_input.SetValue(self.api_config['desc'])
            
        # 添加提示文本
        desc_hint = wx.StaticText(self, label=T("Hint: Description supports multi-line text, will be saved in markdown format"))
        desc_hint.SetFont(wx.Font(9, wx.FONTFAMILY_DEFAULT, wx.FONTSTYLE_NORMAL, wx.FONTWEIGHT_NORMAL))
        
        desc_sizer.Add(desc_label, 0, wx.ALL, 5)
        desc_sizer.Add(self.desc_input, 1, wx.EXPAND | wx.ALL, 5)
        desc_sizer.Add(desc_hint, 0, wx.LEFT | wx.BOTTOM, 5)
        
        main_sizer.Add(desc_sizer, 1, wx.EXPAND | wx.ALL, 10)
        
        # 按钮
        button_sizer = wx.BoxSizer(wx.HORIZONTAL)
        save_button = wx.Button(self, wx.ID_OK, T("Save and Apply"))
        cancel_button = wx.Button(self, wx.ID_CANCEL, T("Cancel"))
        
        button_sizer.Add(save_button, 0, wx.ALL, 5)
        button_sizer.Add(cancel_button, 0, wx.ALL, 5)
        
        main_sizer.Add(button_sizer, 0, wx.ALIGN_CENTER | wx.ALL, 10)
        
        self.SetSizer(main_sizer)
        self.Layout()
        
        # 设置最小窗口大小
        self.SetMinSize((600, 500)) 
    
    def add_env_control(self, key_name="", key_value=None):
        """添加一个环境变量控制项"""
        if len(self.env_controls) >= 2:
            return
            
        if key_value is None:
            key_value = ["", T("API key description")]
        
        env_item = wx.Panel(self.env_panel)
        item_sizer = wx.BoxSizer(wx.HORIZONTAL)
        
        # 变量名
        key_name_label = wx.StaticText(env_item, label=f"{T('Variable Name')}:")
        key_name_input = wx.TextCtrl(env_item, size=(150, -1))
        key_name_input.SetValue(key_name)
        
        # 值
        key_value_label = wx.StaticText(env_item, label=f"{T('Value')}:")
        key_value_input = wx.TextCtrl(env_item, size=(150, -1))
        if isinstance(key_value, list) and len(key_value) > 0:
            key_value_input.SetValue(key_value[0])
        
        # 描述
        key_desc_label = wx.StaticText(env_item, label=f"{T('Description')}:")
        key_desc_input = wx.TextCtrl(env_item, size=(150, -1))
        if isinstance(key_value, list) and len(key_value) > 1:
            key_desc_input.SetValue(key_value[1])
        
        # 删除按钮
        delete_button = wx.Button(env_item, label=T("Remove"))
        delete_button.Bind(wx.EVT_BUTTON, lambda evt, item=env_item: self.remove_env_item(item))
        
        # 添加到布局
        item_sizer.Add(key_name_label, 0, wx.ALL | wx.ALIGN_CENTER_VERTICAL, 5)
        item_sizer.Add(key_name_input, 0, wx.ALL | wx.ALIGN_CENTER_VERTICAL, 5)
        item_sizer.Add(key_value_label, 0, wx.ALL | wx.ALIGN_CENTER_VERTICAL, 5)
        item_sizer.Add(key_value_input, 0, wx.ALL | wx.ALIGN_CENTER_VERTICAL, 5)
        item_sizer.Add(key_desc_label, 0, wx.ALL | wx.ALIGN_CENTER_VERTICAL, 5)
        item_sizer.Add(key_desc_input, 0, wx.ALL | wx.ALIGN_CENTER_VERTICAL, 5)
        item_sizer.Add(delete_button, 0, wx.ALL | wx.ALIGN_CENTER_VERTICAL, 5)
        
        env_item.SetSizer(item_sizer)
        self.env_sizer.Add(env_item, 0, wx.EXPAND | wx.ALL, 5)
        
        # 添加分隔线
        separator = wx.StaticLine(self.env_panel)
        self.env_sizer.Add(separator, 0, wx.EXPAND | wx.ALL, 5)
        
        self.env_controls.append({
            'panel': env_item,
            'name': key_name_input,
            'value': key_value_input,
            'desc': key_desc_input
        })
        
        self.env_panel.Layout()
        # 确保新添加的控件可见
        self.env_panel.Scroll(0, self.env_sizer.GetSize().GetHeight())
    
    def remove_env_item(self, item):
        """移除环境变量控制项"""
        for i, ctrl in enumerate(self.env_controls):
            if ctrl['panel'] == item:
                item.Destroy()
                self.env_controls.pop(i)
                break
        
        self.env_panel.Layout()
        self.env_panel.SetupScrolling()
    
    def on_add_env(self, event):
        """添加新的环境变量"""
        if len(self.env_controls) >= 2:
            wx.MessageBox(T("You can only add two environment variables at most"), T("Hint"), wx.OK | wx.ICON_INFORMATION)
            return
        self.add_env_control()
    
    def get_api_config(self):
        """获取编辑后的API配置"""
        name = self.name_input.GetValue().strip()
        
        config = {
            "desc": self.desc_input.GetValue()  # 保留原始格式，包括换行
        }
        
        env = {}
        for ctrl in self.env_controls:
            # 获取用户输入的键名
            key_name = ctrl['name'].GetValue().strip()
            
            # 如果键名为空，则跳过
            if not key_name:
                continue
                
            key_value = ctrl['value'].GetValue().strip()
            key_desc = ctrl['desc'].GetValue().strip()
            
            env[key_name] = [key_value, key_desc]

        if env: config['env'] = env        
        return name, config


class ApiDetailsDialog(wx.Dialog):
    """API详情对话框"""
    def __init__(self, parent, api_name, api_config):
        super().__init__(parent, title=f"{T('API Details')}: {api_name}", size=(500, 400))
        
        self.api_name = api_name
        self.api_config = api_config
        self._init_ui()
    
    def _init_ui(self):
        """初始化UI界面"""
        main_sizer = wx.BoxSizer(wx.VERTICAL)
        
        # 创建滚动面板
        scroll_panel = ScrolledPanel(self)
        scroll_sizer = wx.BoxSizer(wx.VERTICAL)
        
        # 描述
        if 'desc' in self.api_config and self.api_config['desc']:
            desc_group = wx.StaticBox(scroll_panel, label=T("Description"))
            desc_sizer = wx.StaticBoxSizer(desc_group, wx.VERTICAL)
            
            desc_text = wx.StaticText(scroll_panel, label=self.api_config['desc'])
            desc_text.Wrap(450)  # 设置自动换行宽度
            
            desc_sizer.Add(desc_text, 0, wx.ALL | wx.EXPAND, 10)
            scroll_sizer.Add(desc_sizer, 0, wx.ALL | wx.EXPAND, 5)
        
        # 查找环境变量键
        envs = self.api_config.get('env')
        if envs:
            env_group = wx.StaticBox(scroll_panel, label=T("Environment Variables"))
            env_sizer = wx.StaticBoxSizer(env_group, wx.VERTICAL)
            
            for i, key_name in enumerate(envs.keys()):
                key_value = envs[key_name]
                display_key = key_name
                
                env_panel = wx.Panel(scroll_panel)
                
                env_box = wx.BoxSizer(wx.VERTICAL)
                
                name_text = wx.StaticText(env_panel, label=f"{T('Variable Name')}: {display_key}")
                name_text.SetFont(wx.Font(10, wx.FONTFAMILY_DEFAULT, wx.FONTSTYLE_NORMAL, wx.FONTWEIGHT_BOLD))
                
                if isinstance(key_value, list):
                    if len(key_value) > 0:
                        # 使用掩码显示API密钥
                        masked_key = key_value[0]
                        if len(masked_key) > 8:
                            masked_key = masked_key[:3] + "..." + masked_key[-3:]
                        value_text = wx.StaticText(env_panel, label=f"{T('Value')}: {masked_key}")
                    
                    if len(key_value) > 1 and key_value[1]:
                        desc_text = wx.StaticText(env_panel, label=f"{T('Description')}: {key_value[1]}")
                        desc_text.Wrap(400)
                
                env_box.Add(name_text, 0, wx.ALL, 5)
                env_box.Add(value_text, 0, wx.ALL, 5)
                if len(key_value) > 1 and key_value[1]:
                    env_box.Add(desc_text, 0, wx.ALL, 5)
                
                env_panel.SetSizer(env_box)
                env_sizer.Add(env_panel, 0, wx.ALL | wx.EXPAND, 5)
                
                # 添加分隔线，除了最后一个
                if i < len(envs) - 1:
                    env_sizer.Add(wx.StaticLine(scroll_panel), 0, wx.EXPAND | wx.ALL, 5)
            
            scroll_sizer.Add(env_sizer, 0, wx.ALL | wx.EXPAND, 5)
        
        scroll_panel.SetSizer(scroll_sizer)
        scroll_panel.SetupScrolling()
        
        main_sizer.Add(scroll_panel, 1, wx.EXPAND | wx.ALL, 10)
        
        # 关闭按钮
        close_button = wx.Button(self, wx.ID_OK, T("Close"))
        main_sizer.Add(close_button, 0, wx.ALIGN_CENTER | wx.ALL, 10)
        
        self.SetSizer(main_sizer)
        self.Layout()


class ApiMarketDialog(wx.Dialog):
    """API市场对话框 - 列表视图"""
    def __init__(self, parent, config_manager):
        super().__init__(parent, title=T("API Market"), size=(800, 600))
        
        self.config_manager = config_manager
        self.log = logger.bind(src="apimarket")

        # 确保获取最新配置
        self.config_manager.reload_config()
        self.settings = config_manager.get_config()
        
        # 复制API配置
        self.api_configs = self.settings.get('api', {})
        
        # 创建界面
        self._init_ui()
    
    def _init_ui(self):
        """初始化UI界面"""
        main_sizer = wx.BoxSizer(wx.VERTICAL)
        
        # 标题
        title_text = wx.StaticText(self, label=T("API Market - Manage Your API Configurations"))
        title_text.SetFont(wx.Font(14, wx.FONTFAMILY_DEFAULT, wx.FONTSTYLE_NORMAL, wx.FONTWEIGHT_BOLD))
        main_sizer.Add(title_text, 0, wx.ALIGN_CENTER | wx.ALL, 10)
        
        # 介绍说明
        desc_text = wx.StaticText(self, label=T("Manage your API configurations here, including adding new APIs, viewing and editing existing ones."))
        main_sizer.Add(desc_text, 0, wx.ALIGN_CENTER | wx.BOTTOM, 10)
        
        # 工具栏
        toolbar_sizer = wx.BoxSizer(wx.HORIZONTAL)
        
        add_button = wx.Button(self, label=T("Add New API"))
        add_button.Bind(wx.EVT_BUTTON, self.on_add_api)
        refresh_button = wx.Button(self, label=T("Refresh List"))
        refresh_button.Bind(wx.EVT_BUTTON, self.on_refresh)
        
        toolbar_sizer.Add(add_button, 0, wx.ALL, 5)
        toolbar_sizer.Add(refresh_button, 0, wx.ALL, 5)
        
        main_sizer.Add(toolbar_sizer, 0, wx.LEFT, 10)
        
        # 创建列表控件
        self.api_list = wx.ListCtrl(self, style=wx.LC_REPORT | wx.BORDER_SUNKEN)
        self.api_list.InsertColumn(0, T("API Name"), width=200)
        self.api_list.InsertColumn(1, T("Number of Keys"), width=100)
        self.api_list.InsertColumn(2, T("Description"), width=450)
        
        # 添加操作提示
        help_text = wx.StaticText(self, label=T("Tip: Right-click on API item to view, edit and delete"))
        help_text.SetFont(wx.Font(9, wx.FONTFAMILY_DEFAULT, wx.FONTSTYLE_ITALIC, wx.FONTWEIGHT_NORMAL))
        
        # 加载API配置到列表
        self.load_api_configs()
        
        main_sizer.Add(self.api_list, 1, wx.EXPAND | wx.ALL, 10)
        main_sizer.Add(help_text, 0, wx.LEFT | wx.BOTTOM, 15)
        
        # 右键菜单绑定
        self.api_list.Bind(wx.EVT_LIST_ITEM_RIGHT_CLICK, self.on_right_click)
        
        # 双击查看详情
        self.api_list.Bind(wx.EVT_LIST_ITEM_ACTIVATED, self.on_item_activated)
        
        # 底部按钮
        button_sizer = wx.BoxSizer(wx.HORIZONTAL)
        save_button = wx.Button(self, label=T("Save and Apply"))
        save_button.Bind(wx.EVT_BUTTON, self.on_save)
        cancel_button = wx.Button(self, wx.ID_CANCEL, T("Cancel"))
        
        button_sizer.Add(save_button, 0, wx.ALL, 5)
        button_sizer.Add(cancel_button, 0, wx.ALL, 5)
        
        main_sizer.Add(button_sizer, 0, wx.ALIGN_CENTER | wx.ALL, 10)
        
        self.SetSizer(main_sizer)
        self.Layout()
    
    def load_api_configs(self):
        """加载API配置到列表"""
        self.api_list.DeleteAllItems()
        if not self.api_configs:
            no_api_item = self.api_list.InsertItem(0, T("No API configured"))
            self.api_list.SetItem(no_api_item, 2, T("Click `Add New API` button to add API configuration"))
            return
        
        idx = 0
        for api_name, api_config in self.api_configs.items():
            item = self.api_list.InsertItem(idx, api_name)
            
            env_count = len(api_config.get('env', {}))
            self.api_list.SetItem(item, 1, str(env_count))
            
            # 添加描述
            desc = api_config.get('desc', '')
            if len(desc) > 60:
                desc = desc[:57] + "..."
            self.api_list.SetItem(item, 2, desc)
            
            idx += 1
    
    def on_item_activated(self, event):
        """双击列表项查看详情"""
        idx = event.GetIndex()
        api_name = self.api_list.GetItemText(idx)
        
        if api_name in self.api_configs:
            self.show_api_details(api_name)
    
    def show_api_details(self, api_name):
        """显示API详情"""
        api_config = self.api_configs.get(api_name)
        if api_config:
            dialog = ApiDetailsDialog(self, api_name, api_config)
            dialog.ShowModal()
            dialog.Destroy()
    
    def on_right_click(self, event):
        """右键菜单"""
        if not self.api_list.GetItemCount():
            return
            
        idx = event.GetIndex()
        if idx == -1:
            return
            
        api_name = self.api_list.GetItemText(idx)
        
        # 如果是"暂无API配置"提示项，不显示菜单
        if api_name == T("No API configured"):
            return
        
        menu = wx.Menu()
        
        view_item = menu.Append(wx.ID_VIEW_DETAILS, T("View Details"))
        edit_item = menu.Append(wx.ID_EDIT, T("Edit"))
        delete_item = menu.Append(wx.ID_DELETE, T("Delete"))
        
        self.Bind(wx.EVT_MENU, lambda evt, name=api_name: self.show_api_details(name), view_item)
        self.Bind(wx.EVT_MENU, lambda evt, name=api_name: self.on_edit_api(name), edit_item)
        self.Bind(wx.EVT_MENU, lambda evt, name=api_name: self.on_delete_api(name), delete_item)
        
        self.PopupMenu(menu)
        menu.Destroy()
    
    def on_add_api(self, event=None):
        """添加新API"""
        dialog = ApiEditDialog(self, is_new=True)
        result = dialog.ShowModal()
        
        if result == wx.ID_OK:
            api_name, api_config = dialog.get_api_config()
            
            if not api_name:
                wx.MessageBox(T("API name cannot be empty"), T("Error"), wx.OK | wx.ICON_ERROR)
                return
            
            if api_name in self.api_configs:
                wx.MessageBox(T("API name already exists"), T("Error"), wx.OK | wx.ICON_ERROR)
                return
            
            self.api_configs[api_name] = api_config
            self.load_api_configs()
        
        dialog.Destroy()
    
    def on_edit_api(self, api_name):
        """编辑API配置"""
        if api_name not in self.api_configs:
            return
            
        api_config = self.api_configs[api_name]
        dialog = ApiEditDialog(self, api_name, api_config, is_new=False)
        result = dialog.ShowModal()
        
        if result == wx.ID_OK:
            _, updated_config = dialog.get_api_config()
            self.api_configs[api_name] = updated_config
            self.load_api_configs()
        
        dialog.Destroy()
    
    def on_delete_api(self, api_name):
        """删除API配置"""
        if api_name not in self.api_configs:
            return
            
        dlg = wx.MessageDialog(
            self, 
            f"{T('Are you sure to delete API')} '{api_name}'?",
            T("Confirm Delete"),
            wx.YES_NO | wx.ICON_QUESTION
        )
        result = dlg.ShowModal()
        dlg.Destroy()
        
        if result == wx.ID_YES:
            del self.api_configs[api_name]
            self.load_api_configs()

            self.config_manager.update_api_config({'api': self.api_configs})
    
    def on_refresh(self, event):
        """刷新API列表"""
        # 重新加载配置
        self.config_manager.reload_config()
        self.settings = self.config_manager.get_config()
        
        # 重新加载API配置
        self.api_configs = self.settings.get('api', {})

        # 更新列表
        self.load_api_configs()

    def on_save(self, event):
        """保存API配置"""
        try:
            self.config_manager.update_sys_config({'api': self.api_configs})
            self.settings = self.config_manager.reload_config()
            
            # 更新父窗口的配置
            if hasattr(self.Parent, 'tm'):
                self.Parent.tm.settings = self.config_manager.get_config()
                if hasattr(self.Parent.tm, 'config'):
                    self.Parent.tm.config = self.config_manager.get_config()
            
            # 显示保存成功的消息
            wx.MessageBox(T("API configuration saved and applied"), T("Success"), wx.OK | wx.ICON_INFORMATION)
            self.EndModal(wx.ID_OK)
        except Exception as e:
            traceback.print_exc()
            wx.MessageBox(f"{T('Failed to save configuration')}: {str(e)}", T("Error"), wx.OK | wx.ICON_ERROR)
            traceback.print_exc() 


#!/usr/bin/env python
# coding: utf-8

from collections import OrderedDict
import platform
import locale
import os
from datetime import date

SYSTEM_PROMPT_TEMPLATE = """
{role_prompt}
{aipy_prompt}
{tips_prompt}
{api_prompt}
{mcp_prompt}
"""

AIPY_PROMPT = """
# 输出内容格式规范
输出内容必须采用结构化的 Markdown 格式，并符合以下规则：

## 多行代码块标记
1. 代码块必须用一对HTML注释标记包围，格式如下：
   - 代码开始：<!-- Block-Start: {"name": "代码块名称", "version": 数字版本号如1/2/3, "path": "该代码块的可选文件路径"} -->
   - 代码本体：用 Markdown 代码块包裹（如 ```python 或 ```html 等)。
   - 代码结束：<!-- Block-End: { "name": 和Block-Start中的name一致 } -->

2. 多个代码块可以使用同一个name，但版本必须不同。版本最高的代码块会被认为是最新的有效版本。

3. `path` 为代码块需要保存为的本地文件路径可以包含目录, 如果是相对路径则默认为相对当前目录或者用户指定目录.

4. 同一个输出消息里可以定义多个代码块。

5. **正确示例：**
<!-- Block-Start: {"name": "abc123", "version": 1, "path": "main.py"} -->
```python
print("hello world")
```
<!-- Block-End: {"name": "abc123"} -->

## 单行命令标记
1. 每次输出中只能包含 **一个** `Cmd-Exec` 标记，用于执行可执行代码块来完成用户的任务：
   - 格式：<!-- Cmd-Exec: {"name": "要执行的代码块 name"} -->
   - 如果不需要执行任何代码，则不要添加 `Cmd-Exec`。
   - 要执行的代码块必需先使用前述多行代码块标记格式单独定义。
   - 如果代码块有多个版本，执行代码块的最新版本。
   - 可以使用 `Cmd-Exec` 执行会话历史中的所有代码块。特别地，如果需要重复执行某个任务，尽量使用 `Cmd-Exec` 执行而不是重复输出代码块。

2. Cmd-Exec 只能用来执行 Python 代码块，不能执行其它语言(如 JSON/CSS/JavaScript等)的代码块。

3. **正确示例：**
<!-- Cmd-Exec: {"name": "abc123"} -->

## 其它   
1. 所有 JSON 内容必须写成**单行紧凑格式**，例如：
   <!-- Block-Start: {"name": "abc123", "path": "main.py", "version": 1} -->

2. 禁止输出代码内容重复的代码块，通过代码块name来引用之前定义过的代码块。

遵循上述规则，生成输出内容。

# 生成Python代码规则
- 确保代码在下述`Python运行环境描述`中描述的运行环境中可以无需修改直接执行
- 实现适当的错误处理，包括但不限于：
  * 文件操作的异常处理
  * 网络请求的超时和连接错误处理
  * 数据处理过程中的类型错误和值错误处理
- 如果需要区分正常和错误信息，可以把错误信息输出到 stderr。
- 不允许执行可能导致 Python 解释器退出的指令，如 exit/quit 等函数，请确保代码中不包含这类操作。

# Python运行环境描述
在标准 Python 运行环境的基础上额外增加了下述功能：
- 一些预装的第三方包
- 全局 `runtime` 对象
- `set_state` 函数：设置当前代码块的执行结果状态，或保存数据到会话中。
- `get_persistent_state` 函数：获取会话中持久化的状态值。

生成 Python 代码时可以直接使用这些额外功能。

## `set_result` 函数
- 定义: `set_result(**kwargs)`
- 参数: 
  - **kwargs: 状态键值对，类型可以为任意Python基本数据类型，如字符串/数字/列表/字典等。
- 用途: 设置当前代码块的运行结果值，作为当前代码块的执行结果反馈。
- 使用示例：
```python
set_result(success=False, reason="Error: 发生了错误") # 设置当前代码块的执行结果状态
set_result(success=True, data={"name": "John", "age": 30}) # 设置当前代码块的执行结果状态
```

## `set_persistent_state` 函数
- 定义: `set_persistent_state(**kwargs)`
- 参数: 
  - **kwargs: 状态键值对，类型可以为任意Python基本数据类型，如字符串/数字/列表/字典等。
- 用途: 设置会话中持久化的状态值。
- 使用示例：
```python
set_persistent_state(data={"name": "John", "age": 30}) # 保存数据到会话中
```

## `get_persistent_state` 函数
- 类型: 函数。
- 参数: 
  - key: 状态键名
- 用途: 获取会话中持久化的状态值。不存在时返回 None。
- 使用示例：
```python
data = get_persistent_state("data")
```

## 预装的第三方包
下述第三方包可以无需安装直接使用：
- `requests`、`numpy`、`pandas`、`matplotlib`、`seaborn`、`bs4`。

其它第三方包，都必需通过下述 runtime 对象的 install_packages 方法申请安装才能使用。

在使用 matplotlib 时，需要根据系统类型选择和设置合适的中文字体，否则图片里中文会乱码导致无法完成客户任务。
示例代码如下：
```python
import platform

system = platform.system().lower()
font_options = {
    'windows': ['Microsoft YaHei', 'SimHei'],
    'darwin': ['Kai', 'Hei'],
    'linux': ['Noto Sans CJK SC', 'WenQuanYi Micro Hei', 'Source Han Sans SC']
}
```

## 全局 runtime 对象
runtime 对象提供一些协助代码完成任务的方法。

### `runtime.get_block_by_name` 方法
- 功能: 获取指定 name 的最新版本的代码块对象
- 定义: `get_block_by_name(code_block_name)`
- 参数: `code_block_name` 为代码块的名称
- 返回值: 代码块对象，如果不存在则返回 None。

返回的代码块对象包含以下属性：
- `name`: 代码块名称
- `version`: 代码块的版本号
- `lang`: 代码块的编程语言
- `code`: 代码块的代码内容
- `path`: 代码块的文件路径（如果之前未指定则为None）

可以修改代码块的 `code` 属性来更新代码内容。

### runtime.install_packages 方法
- 功能: 申请安装完成任务必需的额外模块
- 参数: 一个或多个 PyPi 包名，如：'httpx', 'requests>=2.25'
- 返回值:True 表示成功, False 表示失败

示例如下：
```python
if runtime.install_packages('httpx', 'requests>=2.25'):
    import httpx
```

### runtime.get_env 方法
- 功能: 获取代码运行需要的环境变量，如 API-KEY 等。
- 定义: get_env(name, default=None, *, desc=None)
- 参数: 第一个参数为需要获取的环境变量名称，第二个参数为不存在时的默认返回值，第三个可选字符串参数简要描述需要的是什么。
- 返回值: 环境变量值，返回 None 或空字符串表示未找到。

示例如下：
```python
env_name = '环境变量名称'
env_value = runtime.get_env(env_name, "No env", desc='访问API服务需要')
if not env_value:
    print(f"Error: {env_name} is not set", file=sys.stderr)
else:
    print(f"{env_name} is available")
```

### runtime.display 方法
- 功能: 显示图片
- 定义: display(path="path/to/image.jpg", url="https://www.example.com/image.png")
- 参数: 
  - path: 图片文件路径
  - url: 图片 URL
- 返回值: 无

示例：
```python
runtime.display(path="path/to/image.png")
runtime.display(url="https://www.example.com/image.png")
```

# 代码执行结果反馈
Python代码块的执行结果会通过JSON对象反馈给你，对象包括以下属性：
- `stdout`: 标准输出内容
- `stderr`: 标准错误输出
- `result`: 前述`set_result` 函数设置的当前代码块执行结果
- `errstr`: 异常信息
- `traceback`: 异常堆栈信息
- `block_name`: 执行的代码块名称

注意：
- 如果某个属性为空，它不会出现在反馈中。

收到反馈后，结合代码和反馈数据，做出下一步的决策。
"""

TIPS_PROMPT = """
# 知识点/最佳实践
{tips}
"""

API_PROMPT = """
# 一些 API 信息
下面是用户提供的一些 API 信息，可能有 API_KEY，URL，用途和使用方法等信息。
这些可能对特定任务有用途，你可以根据任务选择性使用。

注意：
1. 这些 API 信息里描述的环境变量必须用 runtime.get_env 方法获取，绝对不能使用 os.getenv 方法。
2. API获取数据失败时，请输出完整的API响应信息，方便调试和分析问题。

{apis}
"""

MCP_PROMPT = """# MCP工具调用规则：

在此环境中，您可以访问一组工具，用于回答用户的问题。您每条消息只能使用一个工具，并将在用户的回复中收到该工具的使用结果。您可以通过循序渐进地使用工具来完成给定任务，每次工具的使用都以前一次工具使用的结果为依据。

## 工具使用格式
需要调用工具时，请返回以下JSON代码，包含调用的工具名称和参数。
{"action": "call_tool", "name": "tool_name", "arguments": {"arg_name": "arg_value", ...}}

请始终遵守此工具使用格式，以确保正确的解析和执行。

## 工具使用示例

以下是一些使用概念工具的示例：
---
User：查询成都本周的天气
Assistant: 我可以使用 maps_weather 工具来计算操作结果。
```json
{"action": "call_tool", "name": "maps_weather", "arguments": {"city": "成都"}}
```
---
User: “以下操作的结果是什么：5 + 3 + 1294.678？”
Assistant: 我可以使用 python_interpreter 工具来计算操作结果。
```json
{"action": "call_tool", "name": "python_interpreter", "arguments": {"code": "5 + 3 + 1294.678"}}
```

## 可用工具

上述示例使用了可能不存在的概念工具。您只能访问以下工具， 以JSON数组形式提供：
{mcp_tools}

## 工具使用规则

以下是您在解决任务时应始终遵循的规则：

1.  始终为工具使用正确的参数。切勿使用变量名作为操作参数，请使用值。
2.  仅在需要时调用工具：如果您不需要信息，请勿调用搜索代理，尝试自行解决任务。
3.  如果不需要调用工具，直接回答问题即可。
4.  切勿重复之前使用完全相同参数的工具调用。
5.  对于工具使用，请务必使用上述示例中所示的 JSON 格式。请勿使用任何其他格式。

"""

def get_system_prompt(tips, api_prompt, user_prompt=None, mcp_tools=""):
    if user_prompt:
        user_prompt = user_prompt.strip()
    prompts = {
        'role_prompt': user_prompt or tips.role.detail,
        'aipy_prompt': AIPY_PROMPT,
        'tips_prompt': '',
        'mcp_prompt': '',
        'api_prompt': API_PROMPT.format(apis=api_prompt)
    }
    if not user_prompt and len(tips) > 0:
        prompts['tips_prompt'] = TIPS_PROMPT.format(tips=str(tips))

    if mcp_tools:
        prompts['mcp_prompt'] = MCP_PROMPT.replace('{mcp_tools}', mcp_tools)

    return SYSTEM_PROMPT_TEMPLATE.format(**prompts)

def get_task_prompt(instruction, gui=False):
    prompt = OrderedDict()
    prompt['task'] = instruction
    prompt['source'] = "User"
    context = OrderedDict()
    context['os_type'] = platform.system()
    context['os_locale'] = locale.getlocale()
    context['os_platform'] = platform.platform()
    context['python_version'] = platform.python_version()
    context['today'] = date.today().isoformat()
    
    if not gui:
        context['TERM'] = os.environ.get('TERM', 'unknown')
        context['LC_TERMINAL'] = os.environ.get('LC_TERMINAL', 'unknown')

    prompt['context'] = context

    constraints = OrderedDict()
    constraints['reply_language'] = "Now, use the exact language of the `task` field for subsequent responses"
    constraints['file_creation_path'] = 'current_directory'
    if gui:
        constraints['matplotlib'] = "DO NOT use plt.show to display picture because I'm using the Agg backend. Save pictures with plt.savefig() and display them with runtime.display()."

    prompt['constraints'] = constraints
    return prompt

def get_results_prompt(results):
    prompt = OrderedDict()
    prompt['message'] = "These are the execution results of the code block/s automatically returned in the order of execution by the runtime environment."
    prompt['source'] = "Runtime Environment"
    prompt['results'] = results
    return prompt

def get_chat_prompt(msg, task):
    prompt = OrderedDict()
    prompt['message'] = msg
    prompt['source'] = "User"

    context = OrderedDict()
    context['initial_task'] = task
    prompt['context'] = context

    constraints = OrderedDict()
    constraints['reply_language'] = "Now, use the exact language of the `message` field for subsequent responses"
    prompt['constraints'] = constraints
    return prompt

def get_mcp_result_prompt(result):
    prompt = OrderedDict()
    prompt['message'] = "The following is the result of the MCP tool call"
    prompt['source'] = "MCP Tool"
    prompt['result'] = result
    return prompt



#! /usr/bin/env python3
# -*- coding: utf-8 -*-
import sys
import subprocess
from abc import ABC, abstractmethod

from loguru import logger

class BaseRuntime(ABC):
    def __init__(self, envs=None):
        self.envs = envs or {}
        self.packages = set()
        self.log = logger.bind(src='runtime')

    def set_env(self, name, value, desc):
        self.envs[name] = (value, desc)

    def ensure_packages(self, *packages, upgrade=False, quiet=False):
        if not packages:
            return True

        packages = list(set(packages) - self.packages)
        if not packages:
            return True
        
        cmd = [sys.executable, "-m", "pip", "install"]
        if upgrade:
            cmd.append("--upgrade")
        if quiet:
            cmd.append("-q")
        cmd.extend(packages)

        try:
            subprocess.check_call(cmd)
            self.packages.update(packages)
            return True
        except subprocess.CalledProcessError:
            self.log.error("依赖安装失败: {}", " ".join(packages))
        
        return False

    def ensure_requirements(self, path="requirements.txt", **kwargs):
        with open(path) as f:
            reqs = [line.strip() for line in f if line.strip() and not line.startswith("#")]
        return self.ensure_packages(*reqs, **kwargs)
    
    @abstractmethod
    def install_packages(self, packages):
        pass

    @abstractmethod
    def get_env(self, name, default=None, *, desc=None):
        pass
    
    @abstractmethod
    def display(self, path=None, url=None):
        pass

    @abstractmethod
    def input(self, prompt=''):
        pass


from ... import T
from .base import BaseCommand
from .utils import print_records

class EnvCommand(BaseCommand):
    name = 'env'
    description = T('Environment operations')

    def add_subcommands(self, subparsers):
        subparsers.add_parser('list', help=T('List environment variables'))

    def cmd_list(self, args):
        rows = self.manager.tm.list_envs()
        print_records(rows)
        
    def cmd(self, args):
        self.cmd_list(args)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
from rich.console import Console
from prompt_toolkit import PromptSession
from prompt_toolkit.history import FileHistory
from prompt_toolkit.styles import Style
from prompt_toolkit.completion import WordCompleter, merge_completers

from ..aipy import TaskManager, ConfigManager, CONFIG_DIR
from .. import T, set_lang, __version__
from ..config import LLMConfig
from ..aipy.wizard import config_llm
from .command import CommandManager, TaskCommandManager

STYLE_MAIN = {
    'completion-menu.completion': 'bg:#000000 #ffffff',
    'completion-menu.completion.current': 'bg:#444444 #ffffff',
    'completion-menu.meta': 'bg:#000000 #999999',
    'completion-menu.meta.current': 'bg:#444444 #aaaaaa',
    'prompt': 'green',
}

STYLE_AI = {
    'completion-menu.completion': 'bg:#002244 #ffffff',         # 深蓝背景，白色文本
    'completion-menu.completion.current': 'bg:#005577 #ffffff', # 当前选中，亮蓝
    'completion-menu.meta': 'bg:#002244 #cccccc',               # 补全项的 meta 信息
    'completion-menu.meta.current': 'bg:#005577 #eeeeee',       # 当前选中的 meta
    'prompt': 'cyan',
}

class InteractiveConsole():
    def __init__(self, tm, console, settings):
        self.tm = tm
        self.names = tm.client_manager.names
        self.history = FileHistory(str(CONFIG_DIR / ".history"))
        self.console = console
        self.settings = settings
        self.style_main = Style.from_dict(STYLE_MAIN)
        self.style_task = Style.from_dict(STYLE_AI)
        self.command_manager_main = CommandManager(tm)
        self.command_manager_task = TaskCommandManager(tm)
        self.completer_main = self.command_manager_main
        self.completer_task = self.command_manager_task
        self.session = PromptSession(history=self.history, completer=self.completer_main, style=self.style_main)
        self.session_task = PromptSession(history=self.history, completer=self.completer_task, style=self.style_task)
    
    def input_with_possible_multiline(self, prompt_text, task_mode=False):
        session = self.session_task if task_mode else self.session
        first_line = session.prompt([("class:prompt", prompt_text)])
        if not first_line.endswith("\\"):
            return first_line
        # Multi-line input
        lines = [first_line.rstrip("\\")]
        while True:
            next_line = session.prompt([("class:prompt", "... ")])
            if next_line.endswith("\\"):
                lines.append(next_line.rstrip("\\"))
            else:
                lines.append(next_line)
                break
        return "\n".join(lines)

    def run_task(self, task, instruction):
        try:
            task.run(instruction)
        except (EOFError, KeyboardInterrupt):
            pass
        except Exception as e:
            self.console.print_exception()

    def start_task_mode(self, task, instruction):
        self.console.print(f"{T('Enter AI mode, start processing tasks, enter Ctrl+d or /done to end the task')}", style="cyan")
        self.run_task(task, instruction)
        while True:
            try:
                user_input = self.input_with_possible_multiline(">>> ", task_mode=True).strip()
                if len(user_input) < 2: continue
            except (EOFError, KeyboardInterrupt):
                break

            if user_input in ('/done', 'done'):
                break

            if self.command_manager_task.execute(task, user_input):
                continue

            self.run_task(task, user_input)

        try:
            task.done()
        except Exception as e:
            self.console.print_exception()
        self.console.print(f"[{T('Exit AI mode')}]", style="cyan")

    def run(self):
        self.console.print(f"{T('Please enter an instruction or `/help` for more information')}", style="green")
        #self.console.print(f"[cyan]{T('Default')}: [green]{self.names['default']}，[cyan]{T('Enabled')}: [yellow]{' '.join(self.names['enabled'])}")
        tm = self.tm
        while True:
            try:
                user_input = self.input_with_possible_multiline(">> ").strip()
                if len(user_input) < 2:
                    continue

                if user_input.startswith('/'):
                    self.command_manager_main.execute(user_input)
                    continue
                else:
                    task = tm.new_task()
                    self.start_task_mode(task, user_input)
            except (EOFError, KeyboardInterrupt):
                break

def main(args):
    console = Console(record=True)
    console.print(f"[bold cyan]🚀 Python use - AIPython ({__version__}) [[green]https://aipy.app[/green]]")
    conf = ConfigManager(args.config_dir)
    settings = conf.get_config()
    lang = settings.get('lang')
    if lang: set_lang(lang)
    llm_config = LLMConfig(CONFIG_DIR / "config")
    if conf.check_config(gui=True) == 'TrustToken':
        if llm_config.need_config():
            console.print(f"[yellow]{T('Starting LLM Provider Configuration Wizard')}[/yellow]")
            try:
                config = config_llm(llm_config)
            except KeyboardInterrupt:
                console.print(f"[yellow]{T('User cancelled configuration')}[/yellow]")
                return
            if not config:
                return
        settings["llm"] = llm_config.config

    if args.fetch_config:
        conf.fetch_config()
        return

    settings.gui = False
    settings.debug = args.debug
    settings.config_dir = CONFIG_DIR
    
    try:
        tm = TaskManager(settings, console=console)
    except Exception as e:
        console.print_exception()
        return

    update = tm.get_update()
    if update and update.get('has_update'):
        console.print(f"[bold red]🔔 号外❗ {T('Update available')}: {update.get('latest_version')}")
   
    if not tm.client_manager:
        console.print(f"[bold red]{T('No available LLM, please check the configuration file')}")
        return
    
    if args.cmd:
        tm.new_task().run(args.cmd)
        return
    InteractiveConsole(tm, console, settings).run()



import asyncio
import contextlib
import json
import os
import re
import sys
import time
from datetime import timedelta

from loguru import logger
from mcp import ClientSession, StdioServerParameters
from mcp.client.stdio import stdio_client
from mcp.client.sse import sse_client
from mcp.client.streamable_http import streamablehttp_client
from mcp.shared.message import SessionMessage
from mcp.types import JSONRPCMessage
from .. import T

# 猴子补丁：修复第三方库中的 _handle_json_response 方法
# MR如果合并后，可以去掉这段代码 https://github.com/modelcontextprotocol/python-sdk/pull/1057
async def _patched_handle_json_response(
    self,
    response,
    read_stream_writer,
    is_initialization: bool = False,
) -> None:
    """修复后的 JSON 响应处理方法"""
    try:
        content = await response.aread()

        # Parse JSON first to determine structure
        data = json.loads(content)

        if isinstance(data, list):
            messages = [JSONRPCMessage.model_validate(item) for item in data]  # type: ignore
        else:
            message = JSONRPCMessage.model_validate(data)
            messages = [message]

        for message in messages:
            if is_initialization:
                self._maybe_extract_protocol_version_from_message(message)

            session_message = SessionMessage(message)
            await read_stream_writer.send(session_message)
    except Exception as exc:
        logger.error(f"Error parsing JSON response: {exc}")
        await read_stream_writer.send(exc)

# 应用猴子补丁
def _apply_streamable_http_patch():
    """应用 StreamableHTTP 客户端的补丁"""
    try:
        from mcp.client.streamable_http import StreamableHTTPTransport
        # 替换原有方法
        StreamableHTTPTransport._handle_json_response = _patched_handle_json_response
        logger.debug("Applied StreamableHTTP patch for _handle_json_response")
    except ImportError as e:
        logger.warning(f"Failed to apply StreamableHTTP patch: {e}")

# 在模块加载时应用补丁
_apply_streamable_http_patch()


# 预编译正则表达式
CODE_BLOCK_PATTERN = re.compile(r"```(?:json)?\s*([\s\S]*?)\s*```")
JSON_PATTERN = re.compile(r"(\{[^{}]*(?:\{[^{}]*\}[^{}]*)*\})")

def extra_call_tool_blocks(blocks) -> str:
    """
    从代码块列表中提取 MCP call_tool JSON。

    Args:
        blocks (list): CodeBlock 对象列表

    Returns:
        str: 找到的 JSON 字符串，如果没找到则返回空字符串
    """
    if not blocks:
        return ""

    for block in blocks:
        # 检查代码块是否是 JSON 格式
        if hasattr(block, 'lang') and block.lang and block.lang.lower() in ['json', '']:
            # 尝试解析代码块内容
            content = getattr(block, 'code', '') if hasattr(block, 'code') else str(block)
            if content:
                content = content.strip()
                try:
                    data = json.loads(content)
                    # 验证是否是 call_tool 动作
                    if not isinstance(data, dict):
                        continue
                    if "action" not in data or "name" not in data:
                        continue
                    if "arguments" in data and not isinstance(data["arguments"], dict):
                        continue

                    # 返回 JSON 字符串
                    return json.dumps(data, ensure_ascii=False)
                except json.JSONDecodeError:
                    continue

    return ""


def extract_call_tool_str(text) -> str:
    """
    Extract MCP call_tool JSON from text.

    Args:
        text (str): The input text that may contain MCP call_tool JSON.

    Returns:
        str: The JSON str if found and valid, otherwise empty str.
    """

    # 使用预编译的正则模式
    code_blocks = CODE_BLOCK_PATTERN.findall(text)

    # Potential JSON candidates to check
    candidates = code_blocks.copy()

    # 使用预编译的正则模式
    standalone_jsons = JSON_PATTERN.findall(text)
    candidates.extend(standalone_jsons)

    # Try to parse each candidate
    for candidate in candidates:
        candidate = candidate.strip()
        try:
            data = json.loads(candidate)
            # Validate that it's a call_tool action
            if not isinstance(data, dict):
                continue
            if "action" not in data or "name" not in data:
                continue
            if "arguments" in data and not isinstance(data["arguments"], dict):
                continue

            # return json string. not dict
            return json.dumps(data, ensure_ascii=False)
        except json.JSONDecodeError:
            continue

    return ""


class MCPConfigReader:
    def __init__(self, config_path):
        self.config_path = config_path

    def get_mcp_servers(self):
        """读取 mcp.json 文件并返回 MCP 服务器清单，包括禁用的服务器"""
        try:
            with open(self.config_path, "r", encoding="utf-8") as f:
                config = json.load(f)
                # 返回所有服务器配置，包括禁用的
                return config.get("mcpServers", {})
        except FileNotFoundError:
            print(f"Config file not found: {self.config_path}")
            return {}
        except json.JSONDecodeError as e:
            print(T("Error decoding MCP config file {}: {}").format(self.config_path, e))
            return {}


class MCPClientSync:
    def __init__(self, server_config, suppress_output=True):
        self.server_config = server_config
        self.suppress_output = suppress_output
        self.connection_type = self._determine_connection_type()

    def _determine_connection_type(self):
        """确定连接类型：stdio, sse, 或 streamable_http"""
        if "url" in self.server_config:
            # 检查是否明确指定为 streamable_http
            transport_type = self.server_config.get("transport", {}).get("type")
            if transport_type == "streamable_http":
                return "streamable_http"
            # 默认为 SSE
            return "sse"
        else:
            return "stdio"

    @contextlib.contextmanager
    def _suppress_stdout_stderr(self):
        """上下文管理器：临时重定向 stdout 和 stderr 到空设备"""
        if not self.suppress_output:
            yield  # 如果不需要抑制输出，直接返回
            return

        # 保存原始的 stdout 和 stderr
        original_stdout = sys.stdout
        original_stderr = sys.stderr

        try:
            # 使用 os.devnull - 跨平台解决方案
            with open(os.devnull, "w") as devnull:
                sys.stdout = devnull
                sys.stderr = devnull
                yield
        finally:
            # 恢复原始的 stdout 和 stderr
            sys.stdout = original_stdout
            sys.stderr = original_stderr

    def _run_async(self, coro):
        with self._suppress_stdout_stderr():
            try:
                return asyncio.run(coro)
            except Exception as e:
                print(f"Error running async function: {e}")
                raise

    def list_tools(self):
        return self._run_async(self._list_tools())

    def call_tool(self, tool_name, arguments):
        return self._run_async(self._call_tool(tool_name, arguments))

    async def _create_client_session(self):
        """根据连接类型创建相应的客户端会话"""
        if self.connection_type == "stdio":
            # stdio 连接
            server_params = StdioServerParameters(
                command=self.server_config.get("command"),
                args=self.server_config.get("args", []),
                env=self.server_config.get("env"),
            )
            return stdio_client(server_params)
        elif self.connection_type == "sse":
            # SSE 连接
            kargs = {'url' : self.server_config["url"]}
            if "headers" in self.server_config and isinstance(self.server_config["headers"], dict):
                kargs['headers'] = self.server_config["headers"]
            if "timeout" in self.server_config:
                kargs['timeout'] = int(self.server_config["timeout"])
            if "sse_read_timeout" in self.server_config:
                kargs['sse_read_timeout'] = int(self.server_config["sse_read_timeout"])

            return sse_client(**kargs)
        elif self.connection_type == "streamable_http":
            # Streamable HTTP 连接
            kargs = {'url' : self.server_config["url"]}
            if "headers" in self.server_config and isinstance(self.server_config["headers"], dict):
                kargs['headers'] = self.server_config["headers"]
            if "timeout" in self.server_config:
                kargs['timeout'] = timedelta(seconds=int(self.server_config["timeout"]))
            if "sse_read_timeout" in self.server_config:
                kargs['sse_read_timeout'] = timedelta(seconds=int(self.server_config["sse_read_timeout"]))

            return streamablehttp_client(**kargs)
        else:
            raise ValueError(f"Unsupported connection type: {self.connection_type}")

    async def _execute_with_session(self, operation):
        """统一的会话执行方法，处理不同客户端类型的差异"""
        client_session = await self._create_client_session()

        if self.connection_type == "streamable_http":
            async with client_session as (read, write, _):
                async with ClientSession(read, write) as session:
                    await session.initialize()
                    return await operation(session)
        else:
            async with client_session as (read, write):
                async with ClientSession(read, write) as session:
                    await session.initialize()
                    return await operation(session)

    async def _list_tools(self):
        try:
            async def list_operation(session):
                server_tools = await session.list_tools()
                return server_tools.model_dump().get("tools", [])
            
            tools = await self._execute_with_session(list_operation)
            
            if sys.platform == "win32":
                # FIX windows下抛异常的问题
                await asyncio.sleep(3)
            return tools
        except Exception as e:
            logger.exception(f"Failed to list tools: {e}")
            return []

    async def _call_tool(self, tool_name, arguments):
        try:
            async def call_operation(session):
                result = await session.call_tool(tool_name, arguments=arguments)
                return result.model_dump()
            
            ret = await self._execute_with_session(call_operation)
            
            if sys.platform == "win32":
                # FIX windows下抛异常的问题
                await asyncio.sleep(3)
            return ret
        except Exception as e:
            logger.exception(f"Failed to call tool {tool_name}: {e}")
            raise


class MCPToolManager:
    def __init__(self, config_path):
        self.config_path = config_path
        self.config_reader = MCPConfigReader(config_path)
        self.mcp_servers = self.config_reader.get_mcp_servers()
        self._tools_cache = {}  # 缓存已获取的工具列表
        self._inited = False
        # 设置缓存文件路径，与配置文件同目录
        self._cache_file = os.path.join(
            os.path.dirname(config_path), "mcp_tools_cache.json"
        )
        self._config_mtime = (
            os.path.getmtime(config_path) if os.path.exists(config_path) else 0
        )
        # 全局启用/禁用标志，默认禁用
        self._globally_enabled = False
        # 服务器状态缓存，记录每个服务器的启用/禁用状态
        self._server_status = {}
        self._init_server_status()

    def _init_server_status(self):
        """初始化服务器状态，从配置文件中读取初始状态，包括禁用的服务器"""
        
        for server_name, server_config in self.mcp_servers.items():
            # 服务器默认启用，除非配置中明确设置为disabled: true或enabled: false
            is_enabled = not (
                server_config.get("disabled", False)
                or server_config.get("enabled", True) is False
            )
            self._server_status[server_name] = is_enabled

    def _is_cache_valid(self):
        """检查缓存文件是否有效"""
        # 如果缓存文件不存在，缓存无效
        if not os.path.exists(self._cache_file):
            return False

        # 检查缓存文件的创建时间是否超过48小时
        current_time = time.time()
        cache_mtime = os.path.getmtime(self._cache_file)
        if current_time - cache_mtime > 172800:  # 48小时 = 48 * 60 * 60 = 172800秒
            logger.debug("缓存已超过48小时，需要重新获取工具列表")
            return False

        # 如果配置文件修改时间晚于缓存创建时间，缓存无效
        if os.path.exists(self.config_path):
            if self._config_mtime > cache_mtime:
                return False

        return True

    def _save_cache(self):
        """保存工具列表到缓存文件"""
        try:
            cache_data = {
                "config_mtime": self._config_mtime,
                "tools_cache": self._tools_cache,
            }
            with open(self._cache_file, "w", encoding="utf-8") as f:
                json.dump(cache_data, f, ensure_ascii=False, indent=2)
            logger.debug(f"工具列表缓存已保存到 {self._cache_file}")
            return True
        except Exception as e:
            logger.exception(f"保存工具列表缓存失败: {e}")
            return False

    def _load_cache(self):
        """从缓存文件加载工具列表"""
        if not self._is_cache_valid():
            return False

        try:
            with open(self._cache_file, "r", encoding="utf-8") as f:
                cache_data = json.load(f)

            config_mtime = cache_data.get("config_mtime", 0)

            # 检查配置文件修改时间是否与缓存中的一致, mcp.json可能从别的地方复制过来，修改时间异常
            if self._config_mtime != config_mtime:
                logger.debug("配置文件已被修改，需要重新获取工具列表")
                return False

            self._tools_cache = cache_data.get("tools_cache", {})
            self._config_mtime = config_mtime
            self._inited = True
            logger.debug(f"已从缓存加载工具列表: {self._cache_file}")
            return True
        except Exception as e:
            logger.exception(f"加载工具列表缓存失败: {e}")
            return False

    def list_tools(self):
        """返回所有MCP服务器的工具列表
        [{'description': 'Get weather alerts for a US state.\n'
                           '\n'
                           '    Args:\n'
                           '        state: Two-letter US state code (e.g. CA, '
                           'NY)\n'
                           '    ',
            'inputSchema': {'properties': {'state': {'title': 'State',
                                                     'type': 'string'}},
                            'required': ['state'],
                            'title': 'get_alertsArguments',
                            'type': 'object'},
            'name': 'get_alerts',
            'server': 'server1'
            },
        ]
        """
        # 如果全局禁用，直接返回空列表
        if not self._globally_enabled:
            return []
            
        # 尝试从缓存加载
        if self._load_cache():
            # 如果成功加载缓存，直接返回结果
            all_tools = []
            for server_name, tools in self._tools_cache.items():
                # 只返回启用的服务器的工具
                if self._server_status.get(server_name, True):
                    all_tools.extend(tools)
            return all_tools

        # 缓存无效或加载失败，重新获取工具列表
        all_tools = []
        print(T("Initializing MCP server, this may take a while if it's the first load, please wait patiently..."))
        for server_name, server_config in self.mcp_servers.items():
            if server_name not in self._tools_cache:
                try:
                    print("+ Loading MCP", server_name)
                    
                    # 创建 MCPClientSync 实例，传入完整的服务器配置
                    client = MCPClientSync(server_config)
                    tools = client.list_tools()
                    
                    # 为每个工具添加服务器标识
                    for tool in tools:
                        tool["server"] = server_name

                    self._tools_cache[server_name] = tools
                except Exception as e:
                    print(f"Error listing tools for server {server_name}: {e}")
                    self._tools_cache[server_name] = []

            # 添加到总工具列表
            all_tools.extend(self._tools_cache[server_name])
            self._inited = True

        # 保存到缓存
        self._save_cache()

        return all_tools

    def get_tools_prompt(self):
        """
        获取工具列表并转换为 Markdown 格式
        返回格式示例：
        ```markdown
        ## 工具列表
        {"name": "xxxx", "description": "xxxx", "arguments": {"key": "val"}}
        ```
        """
        tools = self.list_tools()
        if not tools:
            return ""

        ret = []
        for tool in tools:
            # 构建工具信息的 JSON 对象
            # 去掉 inputSchema里的additionalProperties、$schema
            input_schema = tool.get("inputSchema", {}).copy()
            if "additionalProperties" in input_schema:
                del input_schema["additionalProperties"]
            if "$schema" in input_schema:
                del input_schema["$schema"]

            ret.append({
                "name": tool.get("name", ""),
                "description": tool.get("description", ""),
                "arguments": input_schema
            })

        # 转换为 JSON 字符串并添加到 Markdown 中
        json_str = json.dumps(ret, ensure_ascii=False)
        return f"```json\n{json_str}\n```\n"

    def _get_all_tools(self):
        """返回所有工具的列表"""
        # 如果全局禁用，直接返回空列表
        if not self._globally_enabled:
            return []

        if self._inited:
            all_tools = []
            for server_name, tools in self._tools_cache.items():
                # 只包含启用的服务器
                if self._server_status.get(server_name, True):
                    for tool in tools:
                        tool["server"] = server_name
                        all_tools.append(tool)
        else:
            all_tools = self.list_tools()
        return all_tools
        
    def get_all_servers(self) -> dict:
        """返回所有服务器的列表及其启用状态"""
        if not self._inited:
            self.list_tools()
        
        # 返回服务器列表及其启用状态
        servers_info = {}
        for server_name, status in self._server_status.items():
            ret = {
                'enabled': status,
                'tools_count': 0
            }
            #if server_name not in self._tools_cache:
            tools = self._tools_cache.get(server_name, [])
            if tools:
                # 如果服务器有工具，则更新工具数量
                ret['tools_count'] = len(tools)
            servers_info[server_name] = ret

        return servers_info

    def call_tool(self, tool_name, arguments):
        """调用指定名称的工具，自动选择最匹配的服务器"""
        # 获取所有工具
        all_tools = self._get_all_tools()
        if not all_tools:
            raise ValueError("No tools available to call.")

        # 查找匹配的工具
        matching_tools = [t for t in all_tools if t["name"] == tool_name]
        if not matching_tools:
            raise ValueError(f"No tool found with name: {tool_name}")

        # 选择参数匹配度最高的工具
        best_match = None
        best_score = -1

        for tool in matching_tools:
            score = 0
            required_params = []

            # 检查工具的输入模式
            if "inputSchema" in tool and "properties" in tool["inputSchema"]:
                properties = tool["inputSchema"]["properties"]
                required_params = tool["inputSchema"].get("required", [])

                # 检查所有必需参数是否提供
                required_provided = all(param in arguments for param in required_params)
                if not required_provided:
                    continue

                # 计算匹配的参数数量
                matching_params = sum(1 for param in arguments if param in properties)
                extra_params = len(arguments) - matching_params

                # 评分：匹配参数越多越好，额外参数越少越好
                score = matching_params - 0.1 * extra_params

            if score > best_score:
                best_score = score
                best_match = tool

        if not best_match:
            raise ValueError(
                f"No suitable tool found for {tool_name} with given arguments"
            )

        # 获取服务器配置
        server_name = best_match["server"]
        server_config = self.mcp_servers[server_name]

        # 创建客户端并调用工具
        client = MCPClientSync(server_config)
        return client.call_tool(tool_name, arguments)

    def process_command(self, args):
        """处理命令行参数，执行相应操作
        
        Args:
            args (list): 命令行参数列表，例如 [], ["enable"], ["disable"], 
                         ["enable", "playwright"], ["disable", "playwright"]
        
        Returns:
            dict: 执行结果
        """
        assert len(args) > 0, "No arguments provided"
        # 第一个参数是action
        action = args[0].lower() or "list"
        
        # 处理全局启用/禁用命令
        if action == "enable" or action == "disable":
            # 检查是全局操作还是针对特定服务器
            if len(args) == 1:
                # 全局启用/禁用
                if action == "enable":
                    self._globally_enabled = True
                    return {
                        "status": "success",
                        "action": "global_enable",
                        "globally_enabled": self._globally_enabled,
                        "servers": self.get_all_servers(),
                        "tools_count": len(self.list_tools())
                    }
                else:  # disable
                    self._globally_enabled = False
                    return {
                        "status": "success",
                        "action": "global_disable",
                        "globally_enabled": self._globally_enabled,
                        "servers": self.get_all_servers(),
                        "tools_count": 0
                    }
            elif len(args) == 2:
                # 针对特定服务器的启用/禁用
                server_name = args[1]

                # 处理特殊情况：星号操作符，对所有服务器执行相同操作
                if server_name == "*":
                    # 遍历所有服务器并设置状态（不改变全局启用/禁用状态）
                    for srv_name in self.mcp_servers.keys():
                        self._server_status[srv_name] = (action == "enable")

                    # 刷新工具列表
                    self.list_tools()
                    return {
                        "status": "success",
                        "action": f"all_servers_{action}",
                        "globally_enabled": self._globally_enabled,
                        "servers": self.get_all_servers(),
                        "tools_count": len(self.list_tools())
                    }

                # 检查服务器是否存在
                if server_name not in self.mcp_servers:
                    return {
                        "status": "error",
                        "message": f"Unknown server: {server_name}"
                    }

                if action == "enable":
                    self._server_status[server_name] = True
                    # 刷新工具列表
                    self.list_tools()
                    return {
                        "status": "success",
                        "action": "server_enable",
                        "server": server_name,
                        "globally_enabled": self._globally_enabled,
                        "servers": self.get_all_servers(),
                        "tools_count": len(self.list_tools())
                    }
                else:  # disable
                    self._server_status[server_name] = False
                    # 刷新工具列表
                    self.list_tools()
                    return {
                        "status": "success",
                        "action": "server_disable",
                        "server": server_name,
                        "globally_enabled": self._globally_enabled,
                        "servers": self.get_all_servers(),
                        "tools_count": len(self.list_tools())
                    }
        elif action == "list":
            return {
                "status": "success",
                "action": "list",
                "globally_enabled": self._globally_enabled,
                "servers": self.get_all_servers(),
                "tools_count": len(self.list_tools())
            }

        # 如果没有匹配任何已知命令
        return {
            "status": "error",
            "message": f"Invalid command: {' '.join(args)}"
        }



import time

import wx
import wx.adv
import threading
import qrcode
import io
from PIL import Image

from .. import T
from ..aipy.trustoken import TrustTokenAPI

class TrustTokenAuthDialog(wx.Dialog):
    """A dialog for TrustToken authentication with QR code display."""
    
    def __init__(self, parent=None, coordinator_url=None, poll_interval=5):
        """
        Initialize the authentication dialog.
        
        Args:
            parent: Parent window
            coordinator_url (str, optional): The coordinator server URL
            poll_interval (int, optional): Polling interval in seconds
        """
        super().__init__(parent, title=T("TrustToken Authentication"), 
                        size=(400, 500), 
                        style=wx.DEFAULT_DIALOG_STYLE )
        
        self.api = TrustTokenAPI(coordinator_url)
        self.poll_interval = poll_interval
        self.request_id = None
        self.polling_thread = None
        self.stop_polling = False
        self.start_time = None
        self.polling_timeout = 310  # 5 minutes and 10 seconds
        
        self._init_ui()
        self.Centre()
        
    def _init_ui(self):
        """Initialize the user interface."""
        main_sizer = wx.BoxSizer(wx.VERTICAL)
        
       # Status text
        self.status_text = wx.StaticText(self, label='')
        main_sizer.Add(self.status_text, 0, wx.TOP | wx.ALIGN_CENTER, 5)

        # QR code display
        self.qr_bitmap = wx.StaticBitmap(self, size=(300, 300))
        main_sizer.Add(self.qr_bitmap, 0, wx.ALL | wx.ALIGN_CENTER, 5)

        self.other_text = wx.adv.HyperlinkCtrl(self, label='扫码绑定AiPy大脑，您也可以配置其它大模型大脑', url='https://d.aipy.app/d/77')
        main_sizer.Add(self.other_text, 0, wx.TOP | wx.ALIGN_CENTER, 5)

        # Progress bar
        self.progress_bar = wx.Gauge(self, range=100)
        main_sizer.Add(self.progress_bar, 0, wx.ALL | wx.EXPAND, 5)
        
        # Time remaining text
        self.time_text = wx.StaticText(self, label='')
        self.time_text.Hide()
        main_sizer.Add(self.time_text, 0, wx.ALL | wx.ALIGN_CENTER, 5)
        
        # Buttons
        self.cancel_button = wx.Button(self, wx.ID_CANCEL, T("Cancel"))
        main_sizer.Add(self.cancel_button, 0, wx.ALIGN_CENTER | wx.ALL, 5)
        
        self.SetSizer(main_sizer)
        
        # Bind events
        self.Bind(wx.EVT_BUTTON, self._on_cancel, self.cancel_button)
        self.Bind(wx.EVT_CLOSE, self._on_close)
        
    def _update_progress(self):
        """Update the progress bar and time remaining."""
        if not self.start_time:
            return
            
        elapsed = time.time() - self.start_time
        if elapsed >= self.polling_timeout:
            progress = 100
            time_remaining = 0
        else:
            progress = int((elapsed / self.polling_timeout) * 100)
            time_remaining = int(self.polling_timeout - elapsed)
            
        wx.CallAfter(self.progress_bar.SetValue, progress)
        wx.CallAfter(self.time_text.SetLabel, T("Time remaining: {} seconds", time_remaining))
        
    def _poll_status(self, save_func):
        """Poll the binding status in a separate thread."""
        self.start_time = time.time()
        self.time_text.Show()
        while not self.stop_polling and time.time() - self.start_time < self.polling_timeout:
            self._update_progress()
            
            data = self.api.check_status(self.request_id)
            if not data:
                time.sleep(self.poll_interval)
                continue
                
            status = data.get('status')
            wx.CallAfter(self._update_status, T("Current status: {}...", T(status)))
            
            if status == 'approved':
                if save_func:
                    save_func(data['secret_token'])
                wx.CallAfter(self.EndModal, wx.ID_OK)
                return True
            elif status == 'expired':
                wx.CallAfter(self._update_status, T("Binding request expired."))
                wx.CallAfter(self.EndModal, wx.ID_CANCEL)
                return False
            elif status == 'pending':
                pass
            else:
                wx.CallAfter(self._update_status, T("Received unknown status: {}", status))
                wx.CallAfter(self.EndModal, wx.ID_CANCEL)
                return False
                
            time.sleep(self.poll_interval)
            
        if not self.stop_polling:
            wx.CallAfter(self._update_status, T("Polling timed out."))
            wx.CallAfter(self.EndModal, wx.ID_CANCEL)
        return False
        
    def _on_cancel(self, event):
        """Handle cancel button click."""
        self.stop_polling = True
        if self.polling_thread:
            self.polling_thread.join()
        self.EndModal(wx.ID_CANCEL)
        
    def _on_close(self, event):
        """Handle dialog close."""
        self.stop_polling = True
        if self.polling_thread:
            self.polling_thread.join()
        event.Skip()
        
    def _update_qr_code(self, url):
        """Update the QR code display with the given URL."""
        try:
            qr = qrcode.QRCode(
                error_correction=qrcode.constants.ERROR_CORRECT_L,
                border=1
            )
            qr.add_data(url)
            qr.make(fit=True)
            
            # Convert QR code to wx.Bitmap
            img = qr.make_image(fill_color="black", back_color="white")
            img_byte_arr = io.BytesIO()
            img.save(img_byte_arr, format='PNG')
            img_byte_arr = img_byte_arr.getvalue()
            
            wx_img = wx.Image(io.BytesIO(img_byte_arr))
            wx_img = wx_img.Scale(300, 300, wx.IMAGE_QUALITY_HIGH)
            self.qr_bitmap.SetBitmap(wx.Bitmap(wx_img))
            self.Layout()
        except Exception as e:
            wx.MessageBox(T("(Could not display QR code: {})", e), T("Error"), wx.OK | wx.ICON_ERROR)
            
    def _update_status(self, status):
        """Update the status text."""
        self.status_text.SetLabel(status)
        self.Layout()
        
    def fetch_token(self, save_func):
        """Start the token fetching process.
        
        Args:
            save_func (callable): Function to save the token when approved.
            
        Returns:
            bool: True if token was successfully fetched and saved, False otherwise.
        """
        self._update_status(T('requesting_binding'))
        data = self.api.request_binding()
        if not data:
            wx.MessageBox(T("Failed to initiate binding request.", None), T("Error"), wx.OK | wx.ICON_ERROR)
            return False
            
        approval_url = data['approval_url']
        self.request_id = data['request_id']
        expires_in = data['expires_in']
        self.polling_timeout = expires_in
        self._update_status(T("Current status: {}...", T("Browser has opened the Trustoken website, please register or login to authorize")))
        self._update_qr_code(approval_url)
        
        # Start polling in a separate thread
        self.polling_thread = threading.Thread(
            target=self._poll_status, args=(save_func,))
        self.polling_thread.daemon = True
        self.polling_thread.start()
        
        # Show the dialog
        result = self.ShowModal()
        return result == wx.ID_OK

if __name__ == "__main__":
    # Test the TrustTokenAuthDialog
    app = wx.App()
    
    def save_token(token):
        print(f"Token received: {token}")
        # Here you would typically save the token to your configuration
        # For example:
        # config_manager.save_tt_config(token)
    
    # Create and show the dialog
    dialog = TrustTokenAuthDialog(None)
    if dialog.fetch_token(save_token):
        print("Authentication successful!")
    else:
        print("Authentication failed or was cancelled.")
    
    app.MainLoop() 


from ... import T
from .base import BaseCommand
from .utils import print_records

class LLMCommand(BaseCommand):
    name = 'llm'
    description = T('LLM operations')

    def add_subcommands(self, subparsers):
        # Commit subcommand
        subparsers.add_parser('list', help=T('List LLM providers'))
        
        tm = self.manager.tm
        names = tm.client_manager.names       
        use_parser = subparsers.add_parser('use', help=T('Use a LLM provider'))
        use_parser.add_argument('--name', choices=names['enabled'], help='Provider name')
        
    def cmd_list(self, args):
        rows = self.manager.tm.list_llms()
        print_records(rows)
        
    def cmd_use(self, args):
        self.manager.tm.use(llm=args.name)
        self.log.info(f'Use {args.name} LLM provider')

    def cmd(self, args):
        self.cmd_list(args)